#include "UnitTemplateClasses.h"

void FileDebug( char *msg, ...  )
{
	FILE	*fp;
	va_list arglist;
	char	msgbuf[32768];

	va_start( arglist, msg );
	vsprintf( &msgbuf[0], msg, arglist );
	va_end( arglist );
	
	fp = fopen("debug.txt","a+");
	fprintf( fp, msgbuf );
	fflush( fp );
	fclose( fp );
}


//-------------------------------------------------------
//-------------------------------------------------------
//
//	DEFENSE CLASS
//
//-------------------------------------------------------
//-------------------------------------------------------
CUnitDefense::CUnitDefense()
{
	// Set internal vars
	vReset();
}

CUnitDefense::~CUnitDefense()
{
}

void CUnitDefense::vReset( void )
{
	m_iType					= 0;
	m_iMissileArmorRating	= 0;
	m_iBulletArmorRating	= 0;
	m_iLaserArmorRating		= 0;
	m_iMeleeArmorRating		= 0;
	m_iHitPoints			= 0;
	m_iRegenRate			= 0;
	strcpy( m_szName, "N/A" );
}

//-------------------------------------------------------
//-------------------------------------------------------
//
//	OFFENSE CLASS
//
//-------------------------------------------------------
//-------------------------------------------------------
CUnitOffense::CUnitOffense()
{
	// Set internal vars
	vReset();
}

CUnitOffense::~CUnitOffense()
{
}

//-------------------------------------------------------
// Set internal variables their default states.  This
// is good for resetting a unit for reuse.
//-------------------------------------------------------
void CUnitOffense::vReset( void )
{
	m_iType					= 0;
	m_iMissileDamageRating	= 0;
	m_iBulletDamageRating	= 0;
	m_iLaserDamageRating	= 0;
	m_iMeleeDamageRating	= 0;
	m_iSplashRadius			= 0;
	m_iRateOfFire			= 0;
	m_fProjectileSpeed		= 0.0f;
	m_iRange				= 0;
	strcpy( m_szName, "N/A" );
}

//-------------------------------------------------------
//-------------------------------------------------------
//
//	MOVEMENT CLASS
//
//-------------------------------------------------------
//-------------------------------------------------------
CUnitMovement::CUnitMovement()
{
	// Set internal vars
	vReset();
}

CUnitMovement::~CUnitMovement()
{
}

//-------------------------------------------------------
// Set internal variables their default states.  This
// is good for resetting a unit for reuse.
//-------------------------------------------------------
void CUnitMovement::vReset( void )
{
	m_iType				= 0;
	m_fMovementSpeed	= 0.0f;
	m_iMovementType		= 0;
	m_fAcceleration		= 0.0f;
	m_fDeacceleration	= 0.0f;
	m_fTurnSpeed		= 0.0f;
	strcpy( m_szName, "N/A" );
}

//-------------------------------------------------------
//-------------------------------------------------------
//
//	UNIT CLASS
//
//-------------------------------------------------------
//-------------------------------------------------------
CUnit::CUnit()
{
	// Set internal vars
	vReset();
}

CUnit::~CUnit()
{
}

//-------------------------------------------------------
// Set internal variables their default states.  This
// is good for resetting a unit for reuse.
//-------------------------------------------------------
void CUnit::vReset( void )
{
	m_Defense		= NULL;
	m_Offense1		= NULL;
	m_Offense2		= NULL;
	m_Offense3		= NULL;
	m_Movement		= NULL;
	m_Animation		= NULL;
	m_iCurHitPoints = 0;
	m_fXPos			= 0.0f;
	m_fYPos			= 0.0f;
	m_fRot			= 0.0f;
	m_fScale		= 1.0f;
	m_iUnitID		= -1;
	m_iParentID		= -1;
	strcpy( m_szName, "N/A" );
	m_bActive		= 0;
	m_iOwner		= 0;
	m_iType			= 0;
	m_fCurSpeed		= 0.0f;
	m_iCurAnimFrame	= 0;
	m_iCurAttackFrame = 0;
	m_iCurStillFrame	= 0;
	m_iCurMoveFrame		= 0;
	m_iCurDieFrame		= 0;
}

//-------------------------------------------------------
// Set internal pointers to loaded types.
//-------------------------------------------------------
void CUnit::vSetBaseValues( CUnitDefense* ptrDef, 
							CUnitOffense* ptrOff1, 
							CUnitOffense* ptrOff2, 
							CUnitOffense* ptrOff3, 
							CUnitMovement* ptrMove,
							CUnitAnimation* ptrAnimation )
{
	// Point to the passed in class objects
	m_Defense		= ptrDef;
	m_Offense1		= ptrOff1;
	m_Offense2		= ptrOff2;
	m_Offense3		= ptrOff3;
	m_Movement		= ptrMove;
	m_Animation		= ptrAnimation;
}

//-------------------------------------------------------
// Set the global position of the unit
//-------------------------------------------------------
void CUnit::vSetPosition( float fX, float fY )
{
	m_fXPos = fX;
	m_fYPos = fY;
}

//-------------------------------------------------------
//-------------------------------------------------------
//
//	TEXTURE CLASS
//
//-------------------------------------------------------
//-------------------------------------------------------
CTexture::CTexture()
{
	// Set the name to null
	memset( m_szName, 0x00, 64 );
	// Set texture to unloaded state
	m_pTexture = NULL;
}

CTexture::~CTexture()
{
	// Release any memory
	vRelease();
}

void CTexture::vRelease( void )
{
	// Release the texture if it exists in memory
	if( m_pTexture ) {
		m_pTexture->Release();
		m_pTexture = NULL;
	}
}

//-------------------------------------------------------
// Sets the Direct3D device necessary for loading
// textures.
//-------------------------------------------------------
void CTexture::vSetRenderDevice( LPDIRECT3DDEVICE9 pd3d )
{
	m_pd3dDevice = pd3d;
}

//-------------------------------------------------------
// Load the texture from the specified file name
//-------------------------------------------------------
void CTexture::vLoad( char *szName )
{
	// Store the filename
	strcpy( m_szName, szName );
	// Load the texture
	D3DXCreateTextureFromFile( m_pd3dDevice, m_szName, &m_pTexture );
}

//-------------------------------------------------------
//-------------------------------------------------------
//
//	UNIT ANIMATION CLASS
//
//-------------------------------------------------------
//-------------------------------------------------------
CUnitAnimation::CUnitAnimation()
{
	// Set # textures to 0
	m_iTotalTextures = 0;
	// Reset internal vars
	vReset();
}

CUnitAnimation::~CUnitAnimation()
{
	vReset();
}

void CUnitAnimation::vReset( void )
{
	memset( m_szName, 0x00, 64 );
	memset( m_szBitmapPrefix, 0x00, 64 );
	
	if( m_iTotalTextures ) {
		delete [] m_Textures;
		m_Textures			= NULL;
		m_iTotalTextures	= 0;
	}

	m_iNumStillFrames		= 0;
	m_iNumMoveFrames		= 0;
	m_iNumAttackFrames		= 0;
	m_iNumDieFrames			= 0;
	m_iType					= 0;
	m_iStartStillFrames		= 0;
	m_iStartMoveFrames		= 0;
	m_iStartAttackFrames	= 0;
	m_iStartDieFrames		= 0;
}

//-------------------------------------------------------
// Sets the Direct3D device necessary for loading
// textures.
//-------------------------------------------------------
void CUnitAnimation::vSetRenderDevice( LPDIRECT3DDEVICE9 pd3d )
{
	m_pd3dDevice = pd3d;
}

void CUnitAnimation::vLoadTextures( void )
{
	//
	// Load the animations
	//
	int		i, j;
	int		iLocalCount = 0;
	char	szBitmapFileName[ 128 ];

	//
	// Allocate memory for the textures
	//
	m_Textures = new CTexture[	
		(m_iNumStillFrames*(UNITMANAGER_MAXOWNERS+1))+
		(m_iNumMoveFrames*(UNITMANAGER_MAXOWNERS+1))+
		(m_iNumAttackFrames*(UNITMANAGER_MAXOWNERS+1))+
		(m_iNumDieFrames*(UNITMANAGER_MAXOWNERS+1)) ];
		
	//
	// Still graphics (fidgit)
	//
	m_iStartStillFrames = 0;
	for( i = 0; i < m_iNumStillFrames; i++ ) {
		for( j = 0; j < UNITMANAGER_MAXOWNERS+1; j++ ) {
			sprintf( szBitmapFileName, "UnitData\\%s%d_%d.tga", 
				m_szBitmapPrefix, iLocalCount, j );
			
			// Set the render device
			m_Textures[ m_iTotalTextures ].vSetRenderDevice( m_pd3dDevice );
			// Load the texture
			m_Textures[ m_iTotalTextures ].vLoad( szBitmapFileName );
			// Increment total # of textures
			m_iTotalTextures++;
		}
		iLocalCount++;
	}
	
	//
	// Move graphics
	//
	m_iStartMoveFrames = m_iTotalTextures;
	for( i = 0; i < m_iNumMoveFrames; i++ ) {
		for( j = 0; j < UNITMANAGER_MAXOWNERS+1; j++ ) {
			sprintf( szBitmapFileName, "UnitData\\%s%d_%d.tga", 
				m_szBitmapPrefix, iLocalCount, j );
			
			// Set the render device
			m_Textures[ m_iTotalTextures ].vSetRenderDevice( m_pd3dDevice );
			// Load the texture
			m_Textures[ m_iTotalTextures ].vLoad( szBitmapFileName );
			// Increment total # of textures
			m_iTotalTextures++;
		}
		iLocalCount++;
	}
	
	//
	// Attack graphics
	//
	m_iStartAttackFrames = m_iTotalTextures;
	for( i = 0; i < m_iNumAttackFrames; i++ ) {
		for( j = 0; j < UNITMANAGER_MAXOWNERS+1; j++ ) {
			sprintf( szBitmapFileName, "UnitData\\%s%d_%d.tga", 
				m_szBitmapPrefix, iLocalCount, j );
			
			// Set the render device
			m_Textures[ m_iTotalTextures ].vSetRenderDevice( m_pd3dDevice );
			// Load the texture
			m_Textures[ m_iTotalTextures ].vLoad( szBitmapFileName );
			// Increment total # of textures
			m_iTotalTextures++;
		}
		iLocalCount++;
	}
	
	//
	// Die graphics
	//
	m_iStartDieFrames = m_iTotalTextures;
	for( i = 0; i < m_iNumDieFrames; i++ ) {
		for( j = 0; j < UNITMANAGER_MAXOWNERS+1; j++ ) {
			sprintf( szBitmapFileName, "UnitData\\%s%d_%d.tga", 
				m_szBitmapPrefix, iLocalCount, j );
			
			// Set the render device
			m_Textures[ m_iTotalTextures ].vSetRenderDevice( m_pd3dDevice );
			// Load the texture
			m_Textures[ m_iTotalTextures ].vLoad( szBitmapFileName );
			// Increment total # of textures
			m_iTotalTextures++;
		}
		iLocalCount++;
	}
}

//-------------------------------------------------------
//-------------------------------------------------------
//
//	UNIT MANAGER CLASS
//
//-------------------------------------------------------
//-------------------------------------------------------
CUnitManager::CUnitManager()
{
	m_DefenseObjs	= NULL;
	m_OffenseObjs	= NULL;
	m_MovementObjs	= NULL;
	m_UnitBaseObjs	= NULL;
	m_UnitObjs		= NULL;
	m_AnimationObjs	= NULL;

	m_iTotalDefObjs			= 0;
	m_iTotalOffObjs			= 0;
	m_iTotalMovObjs			= 0;
	m_iTotalAnimationObjs	= 0;
	m_iTotalUnitBaseObjs	= 0;
	m_iTotalUnitObjs		= 0;

	vReset();
}

CUnitManager::~CUnitManager()
{
	vClearMem();
}

//-------------------------------------------------------
// Free memory allocated for unit base types
//-------------------------------------------------------
void CUnitManager::vClearMem( void ) 
{
	if( m_DefenseObjs ) {
		delete [] m_DefenseObjs;
		m_DefenseObjs = NULL;
	}
	if( m_OffenseObjs ) {
		delete [] m_OffenseObjs;
		m_OffenseObjs = NULL;
	}
	if( m_MovementObjs ) {
		delete [] m_MovementObjs;
		m_MovementObjs = NULL;
	}
	if( m_AnimationObjs ) {
		delete [] m_AnimationObjs;
		m_AnimationObjs = NULL;
	}
	if( m_UnitBaseObjs ) {
		delete [] m_UnitBaseObjs;
		m_UnitBaseObjs = NULL;
	}
	if( m_UnitObjs ) {
		delete [] m_UnitObjs;
		m_UnitObjs = NULL;
	}

	m_iTotalDefObjs			= 0;
	m_iTotalOffObjs			= 0;
	m_iTotalMovObjs			= 0;
	m_iTotalAnimationObjs	= 0;
	m_iTotalUnitBaseObjs	= 0;
	m_iTotalUnitObjs		= 0;
}

//-------------------------------------------------------
// Set internal variables their default states.  This
// is good for resetting a unit for reuse.
//-------------------------------------------------------
void CUnitManager::vReset( void )
{
	vClearMem();

	m_DefenseObjs			= new CUnitDefense[ UNITMANAGER_MAXBASEOBJS ];
	m_OffenseObjs			= new CUnitOffense[ UNITMANAGER_MAXBASEOBJS ];
	m_MovementObjs			= new CUnitMovement[ UNITMANAGER_MAXBASEOBJS ];
	m_AnimationObjs			= new CUnitAnimation[ UNITMANAGER_MAXBASEOBJS ];
	m_UnitBaseObjs			= new CUnit[ UNITMANAGER_MAXBASEOBJS ];
	m_UnitObjs				= new CUnit[ UNITMANAGER_MAXUNITS ];
	m_iTotalUnitObjs		= UNITMANAGER_MAXUNITS;
	
	// Set how many each player owns
	for( int i = 0; i < UNITMANAGER_MAXOWNERS; i++ ) {
		m_iOwnerTotal[ i ] = 0;
	}
}

//-------------------------------------------------------
// Import unit defense, offense, move, and base types 
// from a csv (comma delimited) file.
//-------------------------------------------------------
int CUnitManager::iLoadBaseTypes(	char *szDefFileName, 
									char *szOffFileName, 
									char *szMovFileName, 
									char *szUnitFileName,
									char *szAnimFileName )
{
	FILE			*fp;
	char			szTempBuffer[ 512 ];
	char			szValue[ 32 ][ 32 ];
	//char			szBitmapFileName[ 128 ];
	int				iStart, iEnd, iCurPos;
	int				iCurValue = 0;
	CUnitDefense	*ptrDefense;
	CUnitOffense	*ptrOffense1;
	CUnitOffense	*ptrOffense2;
	CUnitOffense	*ptrOffense3;
	CUnitMovement	*ptrMovement;
	CUnitAnimation	*ptrAnimation;

	//------------------------------------------
	//
	// DEFENSE TYPES
	//
	//------------------------------------------
	// Open the base type file
	fp = fopen( szDefFileName, "r" );
	if( fp == NULL ) {
		return( -1 );
	}

	// Pull the header first and toss it out
	fgets( szTempBuffer, 512, fp );
	szTempBuffer[strlen(szTempBuffer)-1] = '\0';

	// Set total objects to 0
	m_iTotalDefObjs = 0;

	// Loop through and read every line
	while( !feof( fp ) ) {
		// Get next line
		fgets( szTempBuffer, 512, fp );

		if( feof( fp ) ) {
			break;
		}

		// Add terminator
		szTempBuffer[strlen(szTempBuffer)-1] = '\0';
		iStart	= 0;
		iEnd	= 0;
		iCurPos = 0;
		iCurValue = 0;

		// Pull out the values
		while( szTempBuffer[ iCurPos ] != '\0' && iCurPos < 512 ) {
			// Check for end of value
			if( szTempBuffer[ iCurPos ] == ',' ) {
				iEnd = iCurPos;
				memset( &szValue[ iCurValue ][ 0 ], 0x00, 32 );
				memcpy( &szValue[ iCurValue ], &szTempBuffer[ iStart ], iEnd-iStart );
				iStart = iEnd+1;
				iCurValue++;
			}
			iCurPos++;
		};
		// Import the last column
		iEnd = iCurPos;
		memset( &szValue[ iCurValue ][ 0 ], 0x00, 32 );
		memcpy( &szValue[ iCurValue ], &szTempBuffer[ iStart ], iEnd-iStart );
		iStart = iEnd+1;
		iCurValue++;

		//
		// Store data
		//

		// ID Type
		m_DefenseObjs[ m_iTotalDefObjs ].m_iType = m_iTotalDefObjs;
		// Name
		strcpy( m_DefenseObjs[ m_iTotalDefObjs ].m_szName, &szValue[ 0 ][ 0 ] );
		// Bullet armor rating
		m_DefenseObjs[ m_iTotalDefObjs ].m_iBulletArmorRating = atoi( &szValue[ 1 ][ 0 ] );
		// Missile armor rating
		m_DefenseObjs[ m_iTotalDefObjs ].m_iMissileArmorRating = atoi( &szValue[ 2 ][ 0 ] );
		// Laser armor rating
		m_DefenseObjs[ m_iTotalDefObjs ].m_iLaserArmorRating = atoi( &szValue[ 3 ][ 0 ] );
		// Melee armor rating
		m_DefenseObjs[ m_iTotalDefObjs ].m_iMeleeArmorRating = atoi( &szValue[ 4 ][ 0 ] );
		// Hit points
		m_DefenseObjs[ m_iTotalDefObjs ].m_iMeleeArmorRating = atoi( &szValue[ 5 ][ 0 ] );
		// Regen rate
		m_DefenseObjs[ m_iTotalDefObjs ].m_iMeleeArmorRating = atoi( &szValue[ 6 ][ 0 ] );

		// Increment objects
		m_iTotalDefObjs++;
	}
	fclose( fp );

	//------------------------------------------
	//
	// OFFENSE TYPES
	//
	//------------------------------------------
	// Open the base type file
	fp = fopen( szOffFileName, "r" );
	if( fp == NULL ) {
		return( -1 );
	}

	// Pull the header first and toss it out
	fgets( szTempBuffer, 512, fp );
	szTempBuffer[strlen(szTempBuffer)-1] = '\0';

	// Set total objects to 0
	m_iTotalOffObjs = 0;

	// Loop through and read every line
	while( !feof( fp ) ) {
		// Get next line
		fgets( szTempBuffer, 512, fp );

		if( feof( fp ) ) {
			break;
		}

		// Add terminator
		szTempBuffer[strlen(szTempBuffer)-1] = '\0';
		iStart	= 0;
		iEnd	= 0;
		iCurPos = 0;
		iCurValue = 0;

		// Pull out the values
		while( szTempBuffer[ iCurPos ] != '\0' && iCurPos < 512 ) {
			// Check for end of value
			if( szTempBuffer[ iCurPos ] == ',' ) {
				iEnd = iCurPos;
				memset( &szValue[ iCurValue ][ 0 ], 0x00, 32 );
				memcpy( &szValue[ iCurValue ], &szTempBuffer[ iStart ], iEnd-iStart );
				iStart = iEnd+1;
				iCurValue++;
			}
			iCurPos++;
		};
		// Import the last column
		iEnd = iCurPos;
		memset( &szValue[ iCurValue ][ 0 ], 0x00, 32 );
		memcpy( &szValue[ iCurValue ], &szTempBuffer[ iStart ], iEnd-iStart );
		iStart = iEnd+1;
		iCurValue++;

		//
		// Store data
		//

		// ID Type
		m_OffenseObjs[ m_iTotalOffObjs ].m_iType = m_iTotalOffObjs;
		// Name
		memset( m_OffenseObjs[ m_iTotalOffObjs ].m_szName, 0x00, 64 );
		strcpy( m_OffenseObjs[ m_iTotalOffObjs ].m_szName, &szValue[ 0 ][ 0 ] );
		// Bullet damage rating
		m_OffenseObjs[ m_iTotalOffObjs ].m_iBulletDamageRating = atoi( &szValue[ 1 ][ 0 ] );
		// Missile damage rating
		m_OffenseObjs[ m_iTotalOffObjs ].m_iMissileDamageRating = atoi( &szValue[ 2 ][ 0 ] );
		// Laser damage rating
		m_OffenseObjs[ m_iTotalOffObjs ].m_iLaserDamageRating = atoi( &szValue[ 3 ][ 0 ] );
		// Melee damage rating
		m_OffenseObjs[ m_iTotalOffObjs ].m_iMeleeDamageRating = atoi( &szValue[ 4 ][ 0 ] );
		// Proj speed
		m_OffenseObjs[ m_iTotalOffObjs ].m_fProjectileSpeed = (float)atof( &szValue[ 5 ][ 0 ] );
		// Range
		m_OffenseObjs[ m_iTotalOffObjs ].m_iRange = atoi( &szValue[ 6 ][ 0 ] );
		// ROF
		m_OffenseObjs[ m_iTotalOffObjs ].m_iRateOfFire = atoi( &szValue[ 7 ][ 0 ] );
		// Splash Radius
		m_OffenseObjs[ m_iTotalOffObjs ].m_iSplashRadius = atoi( &szValue[ 8 ][ 0 ] );

		// Increment objects
		m_iTotalOffObjs++;
	}
	fclose( fp );

	//------------------------------------------
	//
	// MOVEMENT TYPES
	//
	//------------------------------------------
	// Open the base type file
	fp = fopen( szMovFileName, "r" );
	if( fp == NULL ) {
		return( -1 );
	}

	// Pull the header first and toss it out
	fgets( szTempBuffer, 512, fp );
	szTempBuffer[strlen(szTempBuffer)-1] = '\0';

	// Set total objects to 0
	m_iTotalMovObjs = 0;

	// Loop through and read every line
	while( !feof( fp ) ) {
		// Get next line
		fgets( szTempBuffer, 512, fp );

		if( feof( fp ) ) {
			break;
		}

		// Add terminator
		szTempBuffer[strlen(szTempBuffer)-1] = '\0';
		iStart	= 0;
		iEnd	= 0;
		iCurPos = 0;
		iCurValue = 0;

		// Pull out the values
		while( szTempBuffer[ iCurPos ] != '\0' && iCurPos < 512 ) {
			// Check for end of value
			if( szTempBuffer[ iCurPos ] == ',' ) {
				iEnd = iCurPos;
				memset( &szValue[ iCurValue ][ 0 ], 0x00, 32 );
				memcpy( &szValue[ iCurValue ], &szTempBuffer[ iStart ], iEnd-iStart );
				iStart = iEnd+1;
				iCurValue++;
			}
			iCurPos++;
		};
		// Import the last column
		iEnd = iCurPos;
		memset( &szValue[ iCurValue ][ 0 ], 0x00, 32 );
		memcpy( &szValue[ iCurValue ], &szTempBuffer[ iStart ], iEnd-iStart );
		iStart = iEnd+1;
		iCurValue++;

		//
		// Store data
		//

		// ID Type
		m_MovementObjs[ m_iTotalMovObjs ].m_iType = m_iTotalMovObjs;
		// Name
		memset( m_MovementObjs[ m_iTotalOffObjs ].m_szName, 0x00, 64 );
		strcpy( m_MovementObjs[ m_iTotalMovObjs ].m_szName, &szValue[ 0 ][ 0 ] );
		// Acceleration
		m_MovementObjs[ m_iTotalMovObjs ].m_fAcceleration = (float)atof( &szValue[ 1 ][ 0 ] );
		// Deacceleration
		m_MovementObjs[ m_iTotalMovObjs ].m_fDeacceleration = (float)atof( &szValue[ 2 ][ 0 ] );
		// Speed
		m_MovementObjs[ m_iTotalMovObjs ].m_fMovementSpeed = (float)atof( &szValue[ 3 ][ 0 ] );
		// Move type
		m_MovementObjs[ m_iTotalMovObjs ].m_iMovementType = atoi( &szValue[ 4 ][ 0 ] );
		// Turn speed
		m_MovementObjs[ m_iTotalMovObjs ].m_fTurnSpeed = (float)atof( &szValue[ 5 ][ 0 ] );

		// Increment objects
		m_iTotalMovObjs++;
	}
	fclose( fp );

	//------------------------------------------
	//
	// ANIMATION TYPES
	//
	//------------------------------------------
	// Open the base type file
	fp = fopen( szAnimFileName, "r" );
	if( fp == NULL ) {
		return( -1 );
	}

	// Pull the header first and toss it out
	fgets( szTempBuffer, 512, fp );
	szTempBuffer[strlen(szTempBuffer)-1] = '\0';

	// Set total objects to 0
	m_iTotalAnimationObjs = 0;

	// Loop through and read every line
	while( !feof( fp ) ) {
		// Get next line
		fgets( szTempBuffer, 512, fp );

		if( feof( fp ) ) {
			break;
		}

		// Add terminator
		szTempBuffer[strlen(szTempBuffer)-1] = '\0';
		iStart	= 0;
		iEnd	= 0;
		iCurPos = 0;
		iCurValue = 0;

		// Pull out the values
		while( szTempBuffer[ iCurPos ] != '\0' && iCurPos < 512 ) {
			// Check for end of value
			if( szTempBuffer[ iCurPos ] == ',' ) {
				iEnd = iCurPos;
				memset( &szValue[ iCurValue ][ 0 ], 0x00, 32 );
				memcpy( &szValue[ iCurValue ], &szTempBuffer[ iStart ], iEnd-iStart );
				iStart = iEnd+1;
				iCurValue++;
			}
			iCurPos++;
		};
		// Import the last column
		iEnd = iCurPos;
		memset( &szValue[ iCurValue ][ 0 ], 0x00, 32 );
		memcpy( &szValue[ iCurValue ], &szTempBuffer[ iStart ], iEnd-iStart );
		iStart = iEnd+1;
		iCurValue++;

		//
		// Store data
		//

		// ID Type
		m_AnimationObjs[ m_iTotalAnimationObjs ].m_iType = m_iTotalAnimationObjs;
		// Name
		memset( m_AnimationObjs[ m_iTotalAnimationObjs ].m_szName, 0x00, 64 );
		strcpy( m_AnimationObjs[ m_iTotalAnimationObjs ].m_szName, &szValue[ 0 ][ 0 ] );
		// Prefix
		memset( m_AnimationObjs[ m_iTotalAnimationObjs ].m_szBitmapPrefix, 0x00, 64 );
		strcpy( m_AnimationObjs[ m_iTotalAnimationObjs ].m_szBitmapPrefix, &szValue[ 1 ][ 0 ] );
		// Number of still frames
		m_AnimationObjs[ m_iTotalAnimationObjs ].m_iNumStillFrames = atoi( &szValue[ 2 ][ 0 ] );
		// Number of move frames
		m_AnimationObjs[ m_iTotalAnimationObjs ].m_iNumMoveFrames = atoi( &szValue[ 3 ][ 0 ] );
		// Number of attack frames
		m_AnimationObjs[ m_iTotalAnimationObjs ].m_iNumAttackFrames = atoi( &szValue[ 4 ][ 0 ] );
		// Number of die frames
		m_AnimationObjs[ m_iTotalAnimationObjs ].m_iNumDieFrames = atoi( &szValue[ 5 ][ 0 ] );
		// Set the render device
		m_AnimationObjs[ m_iTotalAnimationObjs ].vSetRenderDevice( m_pd3dDevice );
		// Load the textures
		m_AnimationObjs[ m_iTotalAnimationObjs ].vLoadTextures();

		// Increment objects
		m_iTotalAnimationObjs++;
	}
	fclose( fp );

	//------------------------------------------
	//
	// UNIT TYPES
	//
	//------------------------------------------

	// Open the base type file
	fp = fopen( szUnitFileName, "r" );
	if( fp == NULL ) {
		return( -1 );
	}

	// Pull the header first and toss it out
	fgets( szTempBuffer, 512, fp );
	szTempBuffer[strlen(szTempBuffer)-1] = '\0';

	// Set total objects to 0
	m_iTotalUnitBaseObjs = 0;

	// Loop through and read every line
	while( !feof( fp ) ) {
		// Get next line
		fgets( szTempBuffer, 512, fp );

		if( feof( fp ) ) {
			break;
		}

		// Add terminator
		szTempBuffer[strlen(szTempBuffer)-1] = '\0';
		iStart	= 0;
		iEnd	= 0;
		iCurPos = 0;
		iCurValue = 0;

		// Pull out the values
		while( szTempBuffer[ iCurPos ] != '\0' && iCurPos < 512 ) {
			// Check for end of value
			if( szTempBuffer[ iCurPos ] == ',' ) {
				iEnd = iCurPos;
				memset( &szValue[ iCurValue ][ 0 ], 0x00, 32 );
				memcpy( &szValue[ iCurValue ], &szTempBuffer[ iStart ], iEnd-iStart );
				iStart = iEnd+1;
				iCurValue++;
			}
			iCurPos++;
		};
		// Import the last column
		iEnd = iCurPos;
		memset( &szValue[ iCurValue ][ 0 ], 0x00, 32 );
		memcpy( &szValue[ iCurValue ], &szTempBuffer[ iStart ], iEnd-iStart );
		iStart = iEnd+1;
		iCurValue++;

		//
		// Store data
		//

		// ID Type
		m_UnitBaseObjs[ m_iTotalUnitBaseObjs ].m_iType = m_iTotalUnitBaseObjs;
		// Name
		memset( m_UnitBaseObjs[ m_iTotalUnitBaseObjs ].m_szName, 0x00, 64 );
		strcpy( m_UnitBaseObjs[ m_iTotalUnitBaseObjs ].m_szName, &szValue[ 0 ][ 0 ] );
		// Defense Type
		ptrDefense = ptrGetDefenseType( &szValue[ 1 ][ 0 ] );
		// Offense Type1
		ptrOffense1 = ptrGetOffenseType( &szValue[ 2 ][ 0 ] );
		// Offense Type2
		ptrOffense2 = ptrGetOffenseType( &szValue[ 3 ][ 0 ] );
		// Offense Type3
		ptrOffense3 = ptrGetOffenseType( &szValue[ 4 ][ 0 ] );
		// Move Type
		ptrMovement = ptrGetMoveType( &szValue[ 5 ][ 0 ] );
		// Animation Type
		ptrAnimation = ptrGetAnimType( &szValue[ 6 ][ 0 ] );
		// Set the base types
		m_UnitBaseObjs[ m_iTotalUnitBaseObjs ].vSetBaseValues(	
			ptrDefense, 
			ptrOffense1,
			ptrOffense2,
			ptrOffense3,
			ptrMovement,
			ptrAnimation );

		// Increment objects
		m_iTotalUnitBaseObjs++;
	}
	fclose( fp );

	return( 1 );	
}

//-------------------------------------------------------
// Given the defense type name, return a pointer to the
// type.
//-------------------------------------------------------
CUnitDefense* CUnitManager::ptrGetDefenseType( char *szName )
{
	int				i;
	CUnitDefense	*ptrUnitDefense = NULL;

	for( i = 0; i < m_iTotalDefObjs; i++ ) {
		if( !stricmp( szName, m_DefenseObjs[ i ].m_szName ) ) {
			ptrUnitDefense = &m_DefenseObjs[ i ];
			return( ptrUnitDefense );	
		}
	}

	return( ptrUnitDefense );
}

//-------------------------------------------------------
// Given the offense type name, return a pointer to the
// type.
//-------------------------------------------------------
CUnitOffense* CUnitManager::ptrGetOffenseType( char *szName )
{
	int				i;
	CUnitOffense	*ptrUnitOffense = NULL;

	for( i = 0; i < m_iTotalOffObjs; i++ ) {
		if( !stricmp( szName, m_OffenseObjs[ i ].m_szName ) ) {
			ptrUnitOffense = &m_OffenseObjs[ i ];
			return( ptrUnitOffense );	
		}
	}

	return( ptrUnitOffense );
}

//-------------------------------------------------------
// Given the movement type name, return a pointer to the
// type.
//-------------------------------------------------------
CUnitMovement* CUnitManager::ptrGetMoveType( char *szName )
{
	int				i;
	CUnitMovement	*ptrUnitMovement = NULL;

	for( i = 0; i < m_iTotalMovObjs; i++ ) {
		if( !stricmp( szName, m_MovementObjs[ i ].m_szName ) ) {
			ptrUnitMovement = &m_MovementObjs[ i ];
			return( ptrUnitMovement );	
		}
	}

	return( ptrUnitMovement );
}

//-------------------------------------------------------
// Given the anim type name, return a pointer to the
// type.
//-------------------------------------------------------
CUnitAnimation* CUnitManager::ptrGetAnimType( char *szName )
{
	int				i;
	CUnitAnimation	*ptrUnitAnim = NULL;

	for( i = 0; i < m_iTotalAnimationObjs; i++ ) {
		if( !stricmp( szName, m_AnimationObjs[ i ].m_szName ) ) {
			ptrUnitAnim = &m_AnimationObjs[ i ];
			return( ptrUnitAnim );	
		}
	}

	return( ptrUnitAnim );
}

//-------------------------------------------------------
// Activate a unit for use in the game
//-------------------------------------------------------
int CUnitManager::iAddUnit( char *szName, int iOwner )
{
	int		i;
	int		iFoundID = -1;

	// Find a matching type
	for( i = 0; i < m_iTotalUnitBaseObjs; i++ ) {
		if( !stricmp( szName, m_UnitBaseObjs[ i ].m_szName ) ) {
			iFoundID = i;
			break;
		}
	}

	// Return out of base type not found
	if( iFoundID == -1 ) {
		return( -1 );
	}

	// Find free unit slot
	for( i = 0; i < m_iTotalUnitObjs; i++ ) {
		// Check for inactive unit
		if( !m_UnitObjs[ i ].m_bActive ) {
			// Activate the unit
			m_UnitObjs[ i ].m_bActive	= 1;
			// Set its internal types
			m_UnitObjs[ i ].vSetBaseValues( m_UnitBaseObjs[ iFoundID ].m_Defense,
											m_UnitBaseObjs[ iFoundID ].m_Offense1,
											m_UnitBaseObjs[ iFoundID ].m_Offense2,
											m_UnitBaseObjs[ iFoundID ].m_Offense3,
											m_UnitBaseObjs[ iFoundID ].m_Movement,
											m_UnitBaseObjs[ iFoundID ].m_Animation );

			// Set the unit type
			m_UnitObjs[ i ].m_iType		= iFoundID;
			// Set the owner of the unit
			m_UnitObjs[ i ].m_iOwner	= iOwner;
			// Increment how many the owner owns
			m_iOwnerTotal[ iOwner ]++;

			return( i );
		}
	}

	return( -1 );
}

//-------------------------------------------------------
// Sets the Direct3D device necessary for loading
// textures.
//-------------------------------------------------------
void CUnitManager::vSetRenderDevice( LPDIRECT3DDEVICE9 pd3d )
{
	m_pd3dDevice = pd3d;
}

//-------------------------------------------------------
// Deactivates a unit and resets its internal values.
//-------------------------------------------------------
void CUnitManager::vRemoveUnit( int iUnitID )
{
	// Deactivate the unit if it is active
	if( m_UnitObjs[ iUnitID ].m_bActive ) {
		// Decrement how many the owner owns
		m_iOwnerTotal[ m_UnitObjs[ iUnitID ].m_iOwner ]--;
		m_UnitObjs[ iUnitID ].m_bActive = 0;
		m_UnitObjs[ iUnitID ].vReset();
	}
}

int CUnitManager::iCountTotalTextures( void )
{
	int iCount = 0;

	for( int i = 0; i < m_iTotalAnimationObjs; i++ ) {
		iCount += m_AnimationObjs[ i ].m_iTotalTextures;
	}
	
	return( iCount );
}
