//-----------------------------------------------------------------------------
// Name: D3D_AnimationEditor
//
// Description: Example code showing how edit animations.
//
// File Function: Main body of the program (main.cpp)
//
// Code: 
//		Copyright (c) 2003 LostLogic Corporation. All rights reserved.
//
// Libraries Required:
//		d3d9.lib dxguid.lib d3dx9dt.lib d3dxof.lib comctl32.lib winmm.lib dinput8.lib
//
// Local Files Required:
//		main.cpp
//		main.h
//		Object3DClass.cpp
//		Object3DClass.h
//		ExceptionClass.cpp
//		ExceptionClass.h
//		C3DAnimation.cpp
//		C3DAnimation.h
//
// DX Files: 
//		Copyright (c) 1997-2001 Microsoft Corporation. All rights reserved.
//-----------------------------------------------------------------------------
#include "main.h"

//--------------------------------------------------------------------------------------
//
// Main windows function, ie main()
//
//--------------------------------------------------------------------------------------
int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance, PSTR szCmdLine, int iCmdShow)
{
	HWND		hWnd;
	MSG			msg;
	WNDCLASSEX	wndclass;
	RECT		rcWindowClient;
	DWORD		dwInputTimer = 0;
	int			iResult = 0;
		
    // Set up window class
	wndclass.cbSize			= sizeof(wndclass);
	wndclass.style			= CS_HREDRAW | CS_VREDRAW;
	wndclass.lpfnWndProc	= fnMessageProcessor;
	wndclass.cbClsExtra		= 0;
	wndclass.cbWndExtra		= 0;
	wndclass.hInstance		= hInstance;
	wndclass.hIcon			= LoadIcon( NULL, IDI_APPLICATION );
	wndclass.hCursor		= LoadCursor( NULL, IDC_ARROW );
	wndclass.hbrBackground	= (HBRUSH) GetStockObject (WHITE_BRUSH);
	wndclass.lpszMenuName	= NULL;
	wndclass.lpszClassName	= "D3D_AnimationEditor";	// Registered Class Name
	wndclass.hIconSm		= LoadIcon( NULL, IDI_APPLICATION );

	// Register the window class
	if( RegisterClassEx( &wndclass ) == NULL ) 
	{
		// The program failed, exit
		exit(1);
	}

	// Create the window
	hWnd = CreateWindowEx(	WS_EX_OVERLAPPEDWINDOW,
							"D3D_AnimationEditor",		// Class Name
							"D3D_AnimationEditor",		// Name Displayed on Title Bar
							WS_OVERLAPPEDWINDOW,
							0,
							0,
							g_iWindowWidth,
							g_iWindowHeight,
							NULL,
							NULL,
							hInstance,
							NULL );

	// Store window handle globally for access later
	g_hWnd = hWnd;
	
	// Display the window
	ShowWindow( hWnd, iCmdShow );

	// Set the global instance of the program
	g_hInstance = hInstance;

	// Figure out the client work area
	GetClientRect( hWnd, &rcWindowClient );

	// Calculate the rendering offsets based on the client size
	g_iXOffset = (g_iWindowWidth-(rcWindowClient.right-rcWindowClient.left));
	g_iYOffset = (g_iWindowHeight-(rcWindowClient.bottom-rcWindowClient.top));

	// Resize the window to be truely the resolution desired
	SetWindowPos( 
				hWnd, 
				NULL, 
				0,								// X-Pos
				0,								// Y-Pos
				g_iWindowWidth + g_iXOffset,	// Width
				g_iWindowHeight + g_iYOffset,	// Height
				NULL);

	// Clear out message structure
	ZeroMemory( &msg, sizeof(msg) );

	// Initialize Direct Input
	iResult = iInitDirectInput();
	if( iResult != INPUTERROR_SUCCESS ) {
		MessageBox( hWnd, "DirectInput Error", "Unable to initialize Direct Input.", MB_ICONERROR );
		vCleanup();
		exit( 1 );
	}
	
	// Initialize DI Keyboard
	iResult = iInitKeyboard(hWnd);
	if( iResult != INPUTERROR_SUCCESS ) {
		MessageBox( hWnd, "DirectInput Error", "Unable to initialize Keyboard.", MB_ICONERROR );
		vCleanup();
		exit( 1 );
	}

	// Initialize Direct3D
    if( SUCCEEDED( InitD3D( hWnd ) ) ) {
			
		// Initialize the virtual buffer for the display quad
		vInitInterfaceObjects();

		// Initialize the toolbar
		vCreateToolbar( hWnd, hInstance );

		// Initialize animation information
		vInitAnimation();
		
		// Initialize lights
		vCreateLights();

        // Enter the message loop
        while( msg.message!=WM_QUIT ) {
            if( PeekMessage( &msg, NULL, 0U, 0U, PM_REMOVE ) ) {
                TranslateMessage( &msg );
                DispatchMessage( &msg );
            }
            else {
				// Check if it is ok to process mouse clicks
				if( timeGetTime() > dwInputTimer ) {
					// Check for input
					vCheckInput();
					dwInputTimer = timeGetTime()+50;
				}
				// Render the scene
				vRender();
			}
        }
    }
	// Clean up everything and exit the app
    vCleanup();
    UnregisterClass( "D3D_AnimationEditor", wndclass.hInstance );
    return 0;
}

//--------------------------------------------------------------------------------------
//
// Windows message processor
//
//--------------------------------------------------------------------------------------
LRESULT WINAPI fnMessageProcessor( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
	char szBuffer[ 32 ];

    switch( msg )
    {
		case WM_DESTROY:
            PostQuitMessage( 0 );
            return 0;

		case WM_LBUTTONDOWN:
			g_iLeftButtonDown = 1;
			vCheckMouse();
			break;

		case WM_LBUTTONUP:
			g_iLeftButtonDown = 0;
			break;

		case WM_COMMAND:
				switch(LOWORD(wParam))
				{
					// Check toolbar controls
					case ID_BUTTON_SAVE:
						//
						// *FOR YOU TO DO* Add the ability
						// to save the animation to a 
						// specified filename.
						//
						animTest.vSave( "RobotIdle" );
						break;
					
					case ID_BUTTON_LOAD:
						//
						// *FOR YOU TO DO* Add the ability
						// to load any animation desired here.
						//
						animTest.vLoad( "RobotIdle" );
						// Update the toolbar information
						vUpdateToolbarStats();
						vRender();
						break;
					
					case ID_BUTTON_NEWFRAME:
						// Create a new frame for the animation
						animTest.vNewFrame();
						SetActiveWindow( g_hWnd );
						// Update the toolbar information
						vUpdateToolbarStats();
						break;

					case ID_BUTTON_LOADOBJ:
						// Load a 3D object into the scene
						vLoadObject();
						break;

					// Start or stop the animation
					case ID_BUTTON_STARTSTOP:
						// If it is active, stop it
						if( g_iAnimActive ) {
							g_iAnimActive = 0;
						}
						// It is not active, start it
						else {
							g_iAnimActive = 1;
						}
						SetActiveWindow( g_hWnd );
						vUpdateToolbarStats();
						break;

					// Cycle to the next object
					case ID_BUTTON_NEXTOBJ:
						// Increment the object count
						g_iCurObj++;
						// If past the end, loop back to the beginning
						if( g_iCurObj >= animTest.m_iNumObjects ) {
							g_iCurObj = 0;
						}
						SetActiveWindow( g_hWnd );
						vUpdateToolbarStats();
						break;

					// Cycle to the prev object
					case ID_BUTTON_PREVOBJ:
						// Decrement the object count
						g_iCurObj--;
						// Check if object count less than zero
						if( g_iCurObj < 0 ) {
							// Make sure there are objects
							if( animTest.m_iNumObjects ) {
								// Set the current object to the end
								g_iCurObj = animTest.m_iNumObjects-1;
							}
							// No objects in scene, go to frame 0
							else {
								g_iCurObj = 0;
							}
						}
						SetActiveWindow( g_hWnd );
						vUpdateToolbarStats();
						break;

					// Cycle to next frame
					case ID_BUTTON_NEXTFRAME:
						animTest.iNextFrame();
						SetActiveWindow( g_hWnd );
						vUpdateToolbarStats();
						break;

					// Cycle to prev frame
					case ID_BUTTON_PREVFRAME:
						animTest.iPrevFrame();
						SetActiveWindow( g_hWnd );
						vUpdateToolbarStats();
						break;

					//
					// EDIT BOXES
					//
					case ID_EDITBOX_X:
						memset( szBuffer, 0x00, 32 );
						GetWindowText( hEDITBOX_X, szBuffer, 32 );
						if( animTest.m_iNumFrames > 0 ) {
							animTest.m_keyFrames[ g_iCurObj ][ animTest.m_iCurFrame ]->m_vecTrans.x = (float)atof( szBuffer );
						}
						break;
					
					case ID_EDITBOX_Y:
						memset( szBuffer, 0x00, 32 );
						GetWindowText( hEDITBOX_Y, szBuffer, 32 );
						if( animTest.m_iNumFrames > 0 ) {
							animTest.m_keyFrames[ g_iCurObj ][ animTest.m_iCurFrame ]->m_vecTrans.y = (float)atof( szBuffer );
						}
						break;

					case ID_EDITBOX_Z:
						memset( szBuffer, 0x00, 32 );
						GetWindowText( hEDITBOX_Z, szBuffer, 32 );
						if( animTest.m_iNumFrames > 0 ) {
							animTest.m_keyFrames[ g_iCurObj ][ animTest.m_iCurFrame ]->m_vecTrans.z = (float)atof( szBuffer );
						}
						break;

					case ID_EDITBOX_XROT:
						memset( szBuffer, 0x00, 32 );
						GetWindowText( hEDITBOX_XROT, szBuffer, 32 );
						if( animTest.m_iNumFrames > 0 ) {
							animTest.m_keyFrames[ g_iCurObj ][ animTest.m_iCurFrame ]->m_vecRot.x = (float)atof( szBuffer );
						}
						break;
					
					case ID_EDITBOX_YROT:
						memset( szBuffer, 0x00, 32 );
						GetWindowText( hEDITBOX_YROT, szBuffer, 32 );
						if( animTest.m_iNumFrames > 0 ) {
							animTest.m_keyFrames[ g_iCurObj ][ animTest.m_iCurFrame ]->m_vecRot.y = (float)atof( szBuffer );
						}
						break;

					case ID_EDITBOX_ZROT:
						memset( szBuffer, 0x00, 32 );
						GetWindowText( hEDITBOX_ZROT, szBuffer, 32 );
						if( animTest.m_iNumFrames > 0 ) {
							animTest.m_keyFrames[ g_iCurObj ][ animTest.m_iCurFrame ]->m_vecRot.z = (float)atof( szBuffer );
						}
						break;
												
					default:
						break;
				}

		default:
			break;
    }
    return DefWindowProc( hWnd, msg, wParam, lParam );
}

//--------------------------------------------------------------------------------------
//
// Function to initialize Direct 3D
//
//--------------------------------------------------------------------------------------
HRESULT InitD3D( HWND hWnd )
{
	D3DPRESENT_PARAMETERS	d3dpp;
	D3DXMATRIX				matproj, matview;
	D3DDISPLAYMODE			d3ddm;

    // Create the D3D object
    if( NULL == ( g_pD3D = Direct3DCreate9( D3D_SDK_VERSION ) ) )
        return E_FAIL;
	
	// Get the current desktop display mode, so we can set up a back
	// buffer of the same format
    if( FAILED( g_pD3D->GetAdapterDisplayMode( D3DADAPTER_DEFAULT, &d3ddm ) ) )
        return E_FAIL;
	
    // Setup the back buffer and swap format
	ZeroMemory( &d3dpp, sizeof(d3dpp) );
	d3dpp.Windowed					= TRUE;
	d3dpp.SwapEffect				= D3DSWAPEFFECT_DISCARD;
	d3dpp.BackBufferFormat			= d3ddm.Format;
	d3dpp.EnableAutoDepthStencil	= true;
	//
	// ** CHANGE THIS IS YOUR DISPLAY FAILS TO INITIALIZE **
	// Try D3DFMT_D16 or D3DFMT_D32
	// 
	d3dpp.AutoDepthStencilFormat	= D3DFMT_D16;

    // Create the D3DDevice
    if( FAILED( g_pD3D->CreateDevice( D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hWnd,
                                      D3DCREATE_HARDWARE_VERTEXPROCESSING,
                                      &d3dpp, &g_pd3dDevice ) ) )
    {
        return E_FAIL;
    }
	
	//---------------------------------------------
	//     Setup 3D View and Rendering States
	//---------------------------------------------
	
	// Set up view area
	g_vecCameraOrigin = D3DXVECTOR3( 0.0f, 8.0, -100.0 );
	g_vecCameraTarget = D3DXVECTOR3( 0.0, 0.0, 0.0 );
	vSetupView( g_vecCameraOrigin, g_vecCameraTarget );

	// Setup Projection 
	D3DXMatrixPerspectiveFovLH( &matproj, D3DX_PI/4, 1.0f, 1.0f, 1000.0f );

	// Tell the device to use the above matrix for projection
	g_pd3dDevice->SetTransform( D3DTS_PROJECTION, &matproj );

	// Set render states
	g_pd3dDevice->SetRenderState( D3DRS_CULLMODE, D3DCULL_CCW );
	g_pd3dDevice->SetRenderState( D3DRS_ZENABLE, TRUE );
	g_pd3dDevice->SetRenderState( D3DRS_ZWRITEENABLE, TRUE );
	g_pd3dDevice->SetRenderState( D3DRS_LIGHTING, TRUE );
	g_pd3dDevice->SetRenderState( D3DRS_DITHERENABLE, TRUE );
	g_pd3dDevice->SetRenderState( D3DRS_SPECULARENABLE, FALSE );

    return S_OK;
}

//--------------------------------------------------------------------------------------
//
// Function to cleanup allocated objects
//
//--------------------------------------------------------------------------------------
void vCleanup( void )
{
	// Free the vertex buffer
    if( g_pVBInterface != NULL )        
        g_pVBInterface->Release();

	// Free the 3D device
    if( g_pd3dDevice != NULL ) 
        g_pd3dDevice->Release();

	// Free Direct 3D
    if( g_pD3D != NULL )       
        g_pD3D->Release();
}

//--------------------------------------------------------------------------------------
//
// Function to draw the 3D scene
//
//--------------------------------------------------------------------------------------
void vRender( void )
{
	RECT	rectText;
	char	szOutput[ 256 ];
	bool	bFrameChanged = 0;
	int		iCFrame;

	// Clear the backbuffer to a blue color
	g_pd3dDevice->Clear(	0, NULL, D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER, 
							D3DCOLOR_RGBA( 200, 250, 255, 255 ), 1.0f, 0 );

	// Begin the scene
	g_pd3dDevice->BeginScene();

	// Set default material
	g_pd3dDevice->SetMaterial( &g_mtrlDefault );

	// Set fill state to solid
	g_pd3dDevice->SetRenderState( D3DRS_FILLMODE, D3DFILL_SOLID  );

	// Render the ground object
	vDraw3DObject(	D3DXVECTOR3( 0.0, 0.0, 0.0 ) , 
					D3DXVECTOR3( 200.0, 200.0, 200.0 ), 
					D3DXVECTOR3( 90.0, 0.0, 0.0 ), 
					0 );

	//
	// Render the 3d object( s )
	//
	if( animTest.m_iNumFrames && animTest.m_iNumObjects ) {
		for( int i = 0; i < animTest.m_iNumObjects; i++ ) {

			// Render the current object in filled mode.  Render
			// everything else in wireframe.
			//
			// *FOR YOU TO DO* Add a button to the toolbar that
			// lets you turn this feature on or off.
			
			// Not current, render in wireframe
			if( i != g_iCurObj ) {
				g_pd3dDevice->SetRenderState( D3DRS_FILLMODE, D3DFILL_WIREFRAME );
			}
			// Current, render as solid
			else {
				g_pd3dDevice->SetRenderState( D3DRS_FILLMODE, D3DFILL_SOLID  );
			}

			// Set current frame
			iCFrame = animTest.m_iCurFrame;
			
			// Render the object using the frame information
			// stored in the animation object.
			animTest.m_objObject[ i ]->vDisplayXYZ( 
				animTest.m_keyFrames[ i ][iCFrame]->m_vecTrans.x,
				animTest.m_keyFrames[ i ][iCFrame]->m_vecTrans.y,
				animTest.m_keyFrames[ i ][iCFrame]->m_vecTrans.z,
				animTest.m_keyFrames[ i ][iCFrame]->m_vecRot.x,
				animTest.m_keyFrames[ i ][iCFrame]->m_vecRot.y,
				animTest.m_keyFrames[ i ][iCFrame]->m_vecRot.z,
				animTest.m_keyFrames[ i ][iCFrame]->m_vecScale.x,
				animTest.m_keyFrames[ i ][iCFrame]->m_vecScale.y,
				animTest.m_keyFrames[ i ][iCFrame]->m_vecScale.z );

			// Animate the object
			if( g_iAnimActive ) {
			animTest.m_lCurTime++;
				// Advance to next frame
				if( animTest.m_lCurTime >= animTest.m_keyFrames[ i ][iCFrame]->m_lTimeDelay ) {
					animTest.m_iCurFrame++;
					bFrameChanged = 1;
					animTest.m_lCurTime = 0;
					// Reset frame to zero
					if( animTest.m_iCurFrame >= animTest.m_iNumFrames ) {
						animTest.m_iCurFrame = 0;
					}
				}
			}
		}
	}

	// Output camera position
	rectText.bottom		= 480;
	rectText.left		= 0;
	rectText.right		= 640;
	rectText.top		= 0;
	sprintf(	szOutput, "CAM: (%.2f, %.2f, %.2f), Cur Obj: %d", 
				g_vecCameraOrigin.x, 
				g_vecCameraOrigin.y, 
				g_vecCameraOrigin.z,
				g_iCurObj );
	pD3DXFont->DrawText(	szOutput, -1, &rectText, 
							DT_LEFT, 
							D3DCOLOR_RGBA(0, 0, 0, 255));

	// Frame count
	rectText.top		= 15;
	sprintf(	szOutput, "OBJ: Frame %d of %d (%d Objects)", 
			animTest.m_iCurFrame+1,
			animTest.m_iNumFrames,
			animTest.m_iNumObjects );
	pD3DXFont->DrawText(	szOutput, -1, &rectText, 
						DT_LEFT, 
						D3DCOLOR_RGBA(0, 0, 0, 255));
	
	// Output frame stats
	if( animTest.m_iNumFrames && animTest.m_iNumObjects ) {

		// Frame translation 
		rectText.top		= 25;
		sprintf(	szOutput, "     (%.2f, %.2f, %.2f) Trans", 
					animTest.m_keyFrames[ g_iCurObj ][animTest.m_iCurFrame]->m_vecTrans.x, 
					animTest.m_keyFrames[ g_iCurObj ][animTest.m_iCurFrame]->m_vecTrans.y, 
					animTest.m_keyFrames[ g_iCurObj ][animTest.m_iCurFrame]->m_vecTrans.z );
		pD3DXFont->DrawText(	szOutput, -1, &rectText, 
								DT_LEFT, 
								D3DCOLOR_RGBA(0, 0, 0, 255));
		// Frame rotation
		rectText.top		= 35;
		sprintf(	szOutput, "     (%.2f, %.2f, %.2f) Rot", 
					animTest.m_keyFrames[ g_iCurObj ][animTest.m_iCurFrame]->m_vecRot.x, 
					animTest.m_keyFrames[ g_iCurObj ][animTest.m_iCurFrame]->m_vecRot.y, 
					animTest.m_keyFrames[ g_iCurObj ][animTest.m_iCurFrame]->m_vecRot.z );
		pD3DXFont->DrawText(	szOutput, -1, &rectText, 
								DT_LEFT, 
								D3DCOLOR_RGBA(0, 0, 0, 255));
		// Frame scale
		rectText.top		= 45;
		sprintf(	szOutput, "     (%.2f, %.2f, %.2f) Scale", 
					animTest.m_keyFrames[ g_iCurObj ][animTest.m_iCurFrame]->m_vecScale.x, 
					animTest.m_keyFrames[ g_iCurObj ][animTest.m_iCurFrame]->m_vecScale.y, 
					animTest.m_keyFrames[ g_iCurObj ][animTest.m_iCurFrame]->m_vecScale.z );
		pD3DXFont->DrawText(	szOutput, -1, &rectText, 
								DT_LEFT, 
								D3DCOLOR_RGBA(0, 0, 0, 255));

	}

	// End the scene
	g_pd3dDevice->EndScene();

	// Present the backbuffer contents to the display
	g_pd3dDevice->Present( NULL, NULL, NULL, NULL );

	// Update toolbar stats in-case frame advanced
	if( bFrameChanged ) {
		vUpdateToolbarStats();
	}
}

//--------------------------------------------------------------------------------------
//
// Function to initialize the global interface object vertex buffer
//
// Uses the following global variables:
//
//		g_pd3dDevice			The Direct3D Device
//		g_pVBInterface			The Vertex buffer for the interface objects
//		CUSTOMVERTEX			The Vertex structure
//		D3DFVF_CUSTOMVERTEX		The Vertex format
//		g_pTexture				The Texture array to hold textures
//
//--------------------------------------------------------------------------------------
void vInitInterfaceObjects( void )
{
	CUSTOMVERTEX	*pVertices;
	DWORD			dwColor = D3DCOLOR_XRGB(255,255,255);

	// Create the interface object buffer
	if( FAILED( g_pd3dDevice->CreateVertexBuffer( 4*sizeof(CUSTOMVERTEX),
		0, D3DFVF_CUSTOMVERTEX,
		D3DPOOL_DEFAULT, &g_pVBInterface, NULL ) ) )
	{
		return;
	}

	// Lock the buffer for editing
	if( FAILED( g_pVBInterface->Lock( 0, 0, (void**)&pVertices, 0 ) ) )
		return;

	//
	// Create a quad made up of 3 vertices (triangle strips)
	//
	// 1          3
	//  X--------X
	//  |x       |
	//  | x      |
	//  |  x     |
	//  |   *    |
	//  |    x   |
	//  |     x  |
	//  |      x |
	//  |       x|
	//  X--------X
	// 0          2
	//
	pVertices[0].position = D3DXVECTOR3( -0.5f, -0.5f, 0.0f );
	pVertices[0].tu       = 0.0f;
	pVertices[0].tv       = 1.0f;
	pVertices[0].vecNorm = D3DXVECTOR3(0.0f,0.0f,1.0f);
	pVertices[0].color	= dwColor;

	pVertices[1].position = D3DXVECTOR3( -0.5f, 0.5f, 0.0f );
	pVertices[1].tu       = 0.0f;
	pVertices[1].tv       = 0.0f;
	pVertices[1].vecNorm = D3DXVECTOR3(0.0f,0.0f,1.0f);
	pVertices[1].color	= dwColor;

	pVertices[2].position = D3DXVECTOR3( 0.5f, -0.5f, 0.0f );
	pVertices[2].tu       = 1.0f;
	pVertices[2].tv       = 1.0f;
	pVertices[2].vecNorm = D3DXVECTOR3(0.0f,0.0f,1.0f);
	pVertices[2].color	= dwColor;

	pVertices[3].position = D3DXVECTOR3( 0.5f, 0.5f, 0.0f );
	pVertices[3].tu       = 1.0f;
	pVertices[3].tv       = 0.0f;
	pVertices[3].vecNorm = D3DXVECTOR3(0.0f,0.0f,1.0f);
	pVertices[3].color	= dwColor;

	// Done editing, unlock the buffer
	g_pVBInterface->Unlock();

	//---------------------------------------------
	//      LOAD TEXTURES
	//---------------------------------------------

	// Load the ground texture
	if( FAILED( D3DXCreateTextureFromFile( 
		g_pd3dDevice, 
		"Data\\Textures\\grid.tga", 
		&g_pTexture[ 0 ] ) ) ) {

		return;
	}

	// Text Font
	hFont = CreateFont( 
		16, 0, 0, 0, 
		0, 0, 0, 0, 0, 
		0, 0, PROOF_QUALITY, 
		0, "fixedsys" ); 

	// Create the Direct3D Font
	D3DXCreateFont( g_pd3dDevice, hFont, &pD3DXFont ); 
};
//--------------------------------------------------------------------------------------
//
// Function to draw a 3D object in 2D space
//
// iXPos    = x-position in screen-space of the top-left corner of the object
// iYPos    = y-position in screen-space of the top-left corner of the object
// fXSize   = Width of the object in screen-space (pixels)
// fYSize   = Height of the object in screen-space (pixels)
// iTexture = Index into the texture array of object to draw
//
//--------------------------------------------------------------------------------------
void vDraw3DObject( D3DXVECTOR3 fPos, D3DXVECTOR3 fSize, D3DXVECTOR3 fRot, int iTexture )
{
	D3DXMATRIX	matWorld, matRotation, matTranslation, matScale;

	// Set default position,scale,rotation
	D3DXMatrixIdentity( &matTranslation );
	D3DXMatrixTranslation( &matScale, fPos.x, fPos.y, fPos.z );
	// Scale the object
	D3DXMatrixScaling( &matScale, fSize.x, fSize.y, fSize.z );
	D3DXMatrixMultiply( &matTranslation, &matTranslation, &matScale );
	// Rotate the object
	D3DXMatrixRotationX( &matRotation, (float)DegToRad( fRot.x ) );
	D3DXMatrixMultiply( &matWorld, &matTranslation, &matRotation );
	
	D3DXMatrixRotationY( &matRotation, (float)DegToRad( fRot.y ) );
	D3DXMatrixMultiply( &matWorld, &matWorld, &matRotation );
	
	D3DXMatrixRotationZ( &matRotation, (float)DegToRad( fRot.z ) );
	D3DXMatrixMultiply( &matWorld, &matWorld, &matRotation );

	// Set default material
	g_pd3dDevice->SetMaterial( &g_mtrlDefault );

	// Set matrix
	g_pd3dDevice->SetTransform( D3DTS_WORLD, &matWorld );
	g_pd3dDevice->SetTexture( 0, g_pTexture[ iTexture ] );
	g_pd3dDevice->SetStreamSource( 0, g_pVBInterface, 0, sizeof( CUSTOMVERTEX ) );
	g_pd3dDevice->SetFVF( D3DFVF_CUSTOMVERTEX );
	g_pd3dDevice->DrawPrimitive( D3DPT_TRIANGLESTRIP, 0, 2 );

	// Dereference texture
	g_pd3dDevice->SetTexture(0, NULL);
}

//-----------------------------------------------------------------------------
//
// Initializes the DirectInput system
//
//-----------------------------------------------------------------------------
int	iInitDirectInput(void)
{
	HRESULT	hReturn;
	
	// Do not try to create Direct Input if already created
	if( !pDI ) {
		// Create a DInput object
		if( FAILED( hReturn = DirectInput8Create( 
			g_hInstance, DIRECTINPUT_VERSION, 
			IID_IDirectInput8, (VOID**)&pDI, NULL ) ) ) {
			
			return( INPUTERROR_NODI );
		}
	}
	else {
		return( INPUTERROR_KEYBOARDEXISTS );
	}
	
	return( INPUTERROR_SUCCESS );
}

//-----------------------------------------------------------------------------
//
// Reads the keyboard data buffer for input actions
//
//-----------------------------------------------------------------------------
int iInitKeyboard(HWND hWnd)
{
	HRESULT			hReturn = 0;
	DIPROPDWORD		dipdw; 
	
	// Don't try to create the keyboard twice
	if( pKeyboard ) {
		return( INPUTERROR_KEYBOARDEXISTS );
	}
	
	// Exit out if no Direct Input interface found
	else if ( !pDI ) {
		return( INPUTERROR_NODI );
	}

	// Obtain an interface to the system keyboard device
	if( FAILED( hReturn = pDI->CreateDevice( 
		GUID_SysKeyboard, 
		&pKeyboard, 
		NULL ) ) ) {

		return( INPUTERROR_NOKEYBOARD );
	}
	
	// Create buffer to hold keyboard data
	ZeroMemory( &dipdw, sizeof( DIPROPDWORD ) );
	dipdw.diph.dwSize		= sizeof( DIPROPDWORD ); 
	dipdw.diph.dwHeaderSize = sizeof( DIPROPHEADER ); 
	dipdw.diph.dwObj		= 0; 
	dipdw.diph.dwHow		= DIPH_DEVICE; 
	dipdw.dwData			= KEYBOARD_BUFFERSIZE;
	
	// Set the size of the buffer
	if( FAILED( hReturn = pKeyboard->SetProperty( 
		DIPROP_BUFFERSIZE, 
		&dipdw.diph ) ) ) {
		
		return( INPUTERROR_NOKEYBOARD );
	}
	 
	// Set the format of the keyboard
	if( FAILED( hReturn = pKeyboard->SetDataFormat( 
		&c_dfDIKeyboard ) ) ) {
		
		return( INPUTERROR_NOKEYBOARD );
	}
	
	// Set the co-operative level to exclusive access
	if( FAILED( hReturn = pKeyboard->SetCooperativeLevel( 
		hWnd, 
		DISCL_NONEXCLUSIVE | DISCL_FOREGROUND 
		) ) ) {
		
		return( INPUTERROR_NOKEYBOARD );
	}
	
	// Acquire the keyboard device
	pKeyboard->Acquire();

	// Get the keyboard layout, this is required
	// for converting scan codes to ascii codes.
	g_Layout = GetKeyboardLayout( 0 );
	
	return( INPUTERROR_SUCCESS );
}

//-----------------------------------------------------------------------------
//
// Reads the keyboard data buffer for input actions
//
//-----------------------------------------------------------------------------
int iReadKeyboard( void )
{
	HRESULT				hr;
	DWORD				dwCurBuffer;
	DIDEVICEOBJECTDATA	didKeyboardBuffer[KEYBOARD_BUFFERSIZE];
	DWORD				dwItems = KEYBOARD_BUFFERSIZE;
	BYTE				byteASCII;

	// Dont try to read the keyboard if the interface or device is invalid
    if( !pKeyboard || !pDI ) {
        return( INPUTERROR_NOKEYBOARD );
	}

	// Read the buffered data
	hr = pKeyboard->GetDeviceData( 
		sizeof( DIDEVICEOBJECTDATA ), 
		didKeyboardBuffer, 
		&dwItems, 
		0 ); 
	
	// Keyboard may have been lost, reacquire it
	if( FAILED( hr ) ) {
		pKeyboard->Acquire();
		return( INPUTERROR_SUCCESS );
	}
	
	// Process data if there is data to read
	if ( dwItems ) {
		// Process the data
		for( dwCurBuffer = 0; dwCurBuffer < dwItems; dwCurBuffer++ ) {
			
			// Default all keys to being "up"
			for( int j = 0; j < 256; j++ ) {
				ascKeys[ j ][ dwCurBuffer ] = 0;
				diks[ j ][ dwCurBuffer ] = 0;
			}
			
			// Map scan-code to ascii code
			byteASCII = Scan2Ascii( didKeyboardBuffer[dwCurBuffer].dwOfs );
			
			// Set key to be down (depressed)
			if( didKeyboardBuffer[ dwCurBuffer ].dwData & 0x80 ) {
				ascKeys[ byteASCII ][ dwCurBuffer ] = 1;
				diks[ didKeyboardBuffer[ dwCurBuffer ].dwOfs ]
					[ dwCurBuffer ] = 1;

				// Check if SHIFT key changed
				if( didKeyboardBuffer[ dwCurBuffer ].dwOfs == DIK_LSHIFT || 
					didKeyboardBuffer[ dwCurBuffer ].dwOfs == DIK_RSHIFT ) {
					g_bShift = 1;
				}
			}
			// Set key to be up
			else {
				ascKeys[ byteASCII ][ dwCurBuffer ] = 0;
				diks[ didKeyboardBuffer[ dwCurBuffer ].dwOfs ]
					[ dwCurBuffer ] = 0;
				
				// Check if SHIFT key changed
				if( didKeyboardBuffer[ dwCurBuffer ].dwOfs == DIK_LSHIFT || 
					didKeyboardBuffer[ dwCurBuffer ].dwOfs == DIK_RSHIFT ) {
					g_bShift = 0;
				}
			}
		}
	} 
	// Return # of items read
    return ( dwItems );
}

//-----------------------------------------------------------------------------
//
// Converts scan codes to ASCII codes
//
//-----------------------------------------------------------------------------
BYTE Scan2Ascii( DWORD scancode )
{
	UINT vk;
	
	// Map the scancode to an ascii code
	vk = MapVirtualKeyEx( scancode, 1, g_Layout);

	// Map to lower-case
	vk = tolower( vk );

	// Return the ascii code
	return( vk );
}

//--------------------------------------------------------------------------------------
//
// Checks input from the keyboard 
// and acts on it accordingly.
//
//--------------------------------------------------------------------------------------
void vCheckInput( void )
{
	//
	// KEYBOARD INPUT
	//
	
	// Read from the keyboard buffer
	int iResult = iReadKeyboard();
	
	// Check how many key presses were returned
	if( iResult ) {
		
		// Loop through result data
		for( int i = 0; i < iResult; i++ ) {

			// Exit the program if the ESC key is hit
			if( diks[ DIK_ESCAPE ][ i ] ) {
				PostQuitMessage( 0 );
			}

			//
			// Camera movements
			//
			if( diks[ DIK_LEFT ][ i ] ) {
				g_vecCameraOrigin.x--;
				g_vecCameraTarget.x--;
				vSetupView( g_vecCameraOrigin, g_vecCameraTarget );
			}
			
			if( diks[ DIK_RIGHT ][ i ] ) {
				g_vecCameraOrigin.x++;
				g_vecCameraTarget.x++;
				vSetupView( g_vecCameraOrigin, g_vecCameraTarget );
			}

			if( diks[ DIK_UP ][ i ] ) {
				g_vecCameraOrigin.y++;
				g_vecCameraTarget.y++;
				vSetupView( g_vecCameraOrigin, g_vecCameraTarget );
			}
			
			if( diks[ DIK_DOWN ][ i ] ) {
				g_vecCameraOrigin.y--;
				g_vecCameraTarget.y--;
				vSetupView( g_vecCameraOrigin, g_vecCameraTarget );
			}

			//
			// Object Movement
			//

			if( animTest.m_iNumFrames > 0 ) {
				// Rotate - Y
				if( diks[ DIK_Q ][ i ] ) {
					animTest.vUpdateRot( 
						g_iCurObj, 
						animTest.m_iCurFrame, 
						D3DXVECTOR3(0.0f, -1.0f, 0.0f) );
				}
				// Rotate + Y
				if( diks[ DIK_E ][ i ] ) {
					animTest.vUpdateRot( 
						g_iCurObj, 
						animTest.m_iCurFrame, 
						D3DXVECTOR3(0.0f, 1.0f, 0.0f) );
				}
				// Rotate - X
				if( diks[ DIK_Z ][ i ] ) {
					animTest.vUpdateRot( 
						g_iCurObj, 
						animTest.m_iCurFrame, 
						D3DXVECTOR3(-1.0f, 0.0f, 0.0f) );
				}
				// Rotate + X
				if( diks[ DIK_C ][ i ] ) {
					animTest.vUpdateRot( 
						g_iCurObj, 
						animTest.m_iCurFrame, 
						D3DXVECTOR3(1.0f, 0.0f, 0.0f) );
				}
				// Rotate - Z
				if( diks[ DIK_F ][ i ] ) {
					animTest.vUpdateRot( 
						g_iCurObj, 
						animTest.m_iCurFrame, 
						D3DXVECTOR3(0.0f, 0.0f, -1.0f) );
				}
				// Rotate + Z
				if( diks[ DIK_G ][ i ] ) {
					animTest.vUpdateRot( 
						g_iCurObj, 
						animTest.m_iCurFrame, 
						D3DXVECTOR3(0.0f, 0.0f, 1.0f) );
				}

				// Trans - Y
				if( diks[ DIK_S ][ i ] ) {
					animTest.vUpdateTrans( 
						g_iCurObj, 
						animTest.m_iCurFrame, 
						D3DXVECTOR3(0.0f, -1.0f, 0.0f) );
				}
				// Trans + Y
				if( diks[ DIK_W ][ i ] ) {
					animTest.vUpdateTrans( 
						g_iCurObj, 
						animTest.m_iCurFrame, 
						D3DXVECTOR3(0.0f, 1.0f, 0.0f) );
				}
				// Trans - X
				if( diks[ DIK_A ][ i ] ) {
					animTest.vUpdateTrans( 
						g_iCurObj, 
						animTest.m_iCurFrame, 
						D3DXVECTOR3(-1.0f, 0.0f, 0.0f) );
				}
				// Trans + X
				if( diks[ DIK_D ][ i ] ) {
					animTest.vUpdateTrans( 
						g_iCurObj, 
						animTest.m_iCurFrame, 
						D3DXVECTOR3(1.0f, 0.0f, 0.0f) );
				}
				// Trans - Z
				if( diks[ DIK_V ][ i ] ) {
					animTest.vUpdateTrans( 
						g_iCurObj, 
						animTest.m_iCurFrame, 
						D3DXVECTOR3(0.0f, 0.0f, -1.0f) );
				}
				// Trans + Z
				if( diks[ DIK_B ][ i ] ) {
					animTest.vUpdateTrans( 
						g_iCurObj, 
						animTest.m_iCurFrame, 
						D3DXVECTOR3(0.0f, 0.0f, 1.0f) );
				}

				//
				vUpdateToolbarStats();
			}
		}
	}
}

//--------------------------------------------------------------------------------------
//
// Creates the toolbar that contains the various commands
// for the editor
//
//--------------------------------------------------------------------------------------
void vCreateToolbar(HWND hwnd, HINSTANCE hinst)
{
	WNDCLASSEX	wcToolBar;

	// Set up and register toolbar window class
	wcToolBar.cbSize		= sizeof(wcToolBar);
	wcToolBar.style			= CS_HREDRAW | CS_VREDRAW;
	wcToolBar.lpfnWndProc	= fnMessageProcessor;
	wcToolBar.cbClsExtra	= 0;
	wcToolBar.cbWndExtra	= 0;
	wcToolBar.hInstance		= hinst;
	wcToolBar.hIcon			= LoadIcon( NULL, IDI_APPLICATION );
	wcToolBar.hCursor		= LoadCursor( NULL, IDC_ARROW );
	wcToolBar.hbrBackground	= (HBRUSH) GetStockObject (COLOR_BACKGROUND);
	wcToolBar.lpszMenuName	= NULL;
	wcToolBar.lpszClassName	= "ToolBar";	// Registered Class Name
	wcToolBar.hIconSm		= LoadIcon( NULL, IDI_APPLICATION );
    RegisterClassEx(&wcToolBar);
    
	// Create toolbar window
	hWndToolBar = CreateWindowEx ( 
		WS_EX_LEFT|WS_EX_TOPMOST|WS_EX_TOOLWINDOW,
		"ToolBar", "ToolBar",
		WS_BORDER | WS_VISIBLE | WS_CAPTION | WS_MINIMIZEBOX,	
		g_iWindowWidth+15 , g_iYOffset, 150, g_iWindowHeight-20, hwnd, NULL, hinst, NULL);

	hBUTTON_SAVE = CreateWindow(
		"BUTTON", "Save Anim",
		WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON,
		5, 410, 135, 20, hWndToolBar, (HMENU)ID_BUTTON_SAVE, hinst, NULL);

	hBUTTON_LOAD = CreateWindow(
		"BUTTON", "Load Anim",
		WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON,
		5, 385, 135, 20, hWndToolBar, (HMENU)ID_BUTTON_LOAD, hinst, NULL);

	hBUTTON_LOADOBJ = CreateWindow(
		"BUTTON", "Load Objects",
		WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON,
		5, 320, 135, 20, hWndToolBar, (HMENU)ID_BUTTON_LOADOBJ, hinst, NULL);

	hBUTTON_PREVOBJ = CreateWindow(
		"BUTTON", "Prev Obj",
		WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON,
		5, 255, 135, 20, hWndToolBar, (HMENU)ID_BUTTON_PREVOBJ, hinst, NULL);

	hBUTTON_NEXTOBJ = CreateWindow(
		"BUTTON", "Next Obj",
		WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON,
		5, 280, 135, 20, hWndToolBar, (HMENU)ID_BUTTON_NEXTOBJ, hinst, NULL);

	hBUTTON_PREVFRAME = CreateWindow(
		"BUTTON", "Prev Frame",
		WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON,
		5, 5, 135, 20, hWndToolBar, (HMENU)ID_BUTTON_PREVFRAME, hinst, NULL);

	hBUTTON_NEXTFRAME = CreateWindow(
		"BUTTON", "Next Frame",
		WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON,
		5, 30, 135, 20, hWndToolBar, (HMENU)ID_BUTTON_NEXTFRAME, hinst, NULL);

	hBUTTON_NEWFRAME = CreateWindow(
		"BUTTON", "New Frame",
		WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON,
		5, 65, 135, 20, hWndToolBar, (HMENU)ID_BUTTON_NEWFRAME, hinst, NULL);

	hBUTTON_STARTSTOP = CreateWindow(
		"BUTTON", "Start / Stop Anim",
		WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON,
		5, 130, 135, 20, hWndToolBar, (HMENU)ID_BUTTON_STARTSTOP, hinst, NULL);

	//
	// Text Entry Fields
	//
	hEDITBOX_X = CreateWindow(
		"EDIT", NULL,
		WS_CHILD | WS_VISIBLE | WS_BORDER | ES_LEFT | ES_AUTOHSCROLL,
		5, 170, 60, 20, hWndToolBar, (HMENU)ID_EDITBOX_X, hinst, NULL);

	hEDITBOX_Y = CreateWindow(
		"EDIT", NULL,
		WS_CHILD | WS_VISIBLE | WS_BORDER | ES_LEFT | ES_AUTOHSCROLL,
		5, 195, 60, 20, hWndToolBar, (HMENU)ID_EDITBOX_Y, hinst, NULL);

	hEDITBOX_Z = CreateWindow(
		"EDIT", NULL,
		WS_CHILD | WS_VISIBLE | WS_BORDER | ES_LEFT | ES_AUTOHSCROLL,
		5, 220, 60, 20, hWndToolBar, (HMENU)ID_EDITBOX_Z, hinst, NULL);

	hEDITBOX_XROT = CreateWindow(
		"EDIT", NULL,
		WS_CHILD | WS_VISIBLE | WS_BORDER | ES_LEFT | ES_AUTOHSCROLL,
		75, 170, 60, 20, hWndToolBar, (HMENU)ID_EDITBOX_XROT, hinst, NULL);

	hEDITBOX_YROT = CreateWindow(
		"EDIT", NULL,
		WS_CHILD | WS_VISIBLE | WS_BORDER | ES_LEFT | ES_AUTOHSCROLL,
		75, 195, 60, 20, hWndToolBar, (HMENU)ID_EDITBOX_YROT, hinst, NULL);

	hEDITBOX_ZROT = CreateWindow(
		"EDIT", NULL,
		WS_CHILD | WS_VISIBLE | WS_BORDER | ES_LEFT | ES_AUTOHSCROLL,
		75, 220, 60, 20, hWndToolBar, (HMENU)ID_EDITBOX_ZROT, hinst, NULL);

	// Activate edit area
	SetActiveWindow( g_hWnd );
}

//--------------------------------------------------------------------------------------
//
// Checks the mouse buttons and reacts accordingly
//
//--------------------------------------------------------------------------------------
void vCheckMouse( void )
{
	RECT		rcWindow;
	POINT		Point;
	int			iMouseX;
	int			iMouseY;

	// Get mouse coordinates
	GetCursorPos( &Point );
	iMouseX = Point.x;
	iMouseY = Point.y;
	
	// Figure out the toolbar work area
	GetWindowRect( hWndToolBar, &rcWindow );

	// Check if mouse within tool-bar window
	if( iMouseX > rcWindow.left &&
		iMouseX < rcWindow.right && 
		iMouseY > rcWindow.top &&
		iMouseY < rcWindow.bottom ) 
	{
		// Adjust mouse coords to be local to toolbar
		iMouseX -= rcWindow.right;
		iMouseY -= rcWindow.top;
	}

	// Check if mouse within edit window
	else {
		GetWindowRect( g_hWnd, &rcWindow );

		if( iMouseX > rcWindow.left &&
			iMouseX < rcWindow.right && 
			iMouseY > rcWindow.top &&
			iMouseY < rcWindow.bottom ) 
		{
			// Adjust mouse coords to be local to edit window
			iMouseX -= rcWindow.left+g_iXOffset;
			iMouseY -= rcWindow.top+g_iYOffset;
		}
	}
}

//--------------------------------------------------------------------------------------
//
// Creates lights for the 3D scene
//
//--------------------------------------------------------------------------------------
void vCreateLights(void)
{
	D3DLIGHT9		d3dLight;

	// Initialize the default scene material
    ZeroMemory( &g_mtrlDefault, sizeof(D3DMATERIAL9) );
    g_mtrlDefault.Ambient.r = 1.0f;
	g_mtrlDefault.Ambient.g = 1.0f;
	g_mtrlDefault.Ambient.b = 1.0f;
	g_mtrlDefault.Ambient.a = 1.0f;
	g_mtrlDefault.Diffuse.r = 1.0f;
	g_mtrlDefault.Diffuse.g = 1.0f;
	g_mtrlDefault.Diffuse.b = 1.0f;
	g_mtrlDefault.Diffuse.a = 1.0f;
	g_pd3dDevice->SetMaterial( &g_mtrlDefault );
	
	// Clear out light structure
	ZeroMemory(&d3dLight, sizeof(D3DLIGHT9));
	d3dLight.Type = D3DLIGHT_POINT;
	d3dLight.Diffuse.r  = 1.0f;
	d3dLight.Diffuse.g  = 1.0f;
	d3dLight.Diffuse.b  = 1.0f;
	d3dLight.Position.x = 0.0f;
	d3dLight.Position.y = 30.0f;
	d3dLight.Position.z = -100.0f;
	d3dLight.Attenuation0 = 1.0f; 
	d3dLight.Attenuation1 = 0.0025f;
	d3dLight.Range      = 1000.0f;
	// Set the active light
    g_pd3dDevice->SetLight( 0, &d3dLight );
	// Enable the light
    g_pd3dDevice->LightEnable( 0, TRUE );
	// Set ambient light
	g_pd3dDevice->SetRenderState( D3DRS_AMBIENT, 0x00606060 );
}

//--------------------------------------------------------------------------------------
//
// Sets up the view matrix for the 3D scene.  This puts the 
// camera in its proper place
//
//--------------------------------------------------------------------------------------
void vSetupView( D3DXVECTOR3 origin, D3DXVECTOR3 target )
{
	D3DXMATRIX	matview;

	// Setup Camera View
	D3DXMatrixLookAtLH( &matview, 
						&origin, 
						&target, 
						&D3DXVECTOR3( 0.0f, 1.0f, 0.0f ) );

	// Tell the device to use the camera view for the viewport
	g_pd3dDevice->SetTransform( D3DTS_VIEW, &matview );
}

//--------------------------------------------------------------------------------------
//
// Updates the text on the toolbar window.  For now this includes
// the translation and rotation data for the current object.
//
// *FOR YOU TO DO* Add input and output for the scale parameters
// of the object.
//
//--------------------------------------------------------------------------------------
void vUpdateToolbarStats( void )
{
	char szBuffer[ 32 ];

	if( animTest.m_iNumFrames > 0 ) {
		// Translation
		sprintf( szBuffer, "%.2f", animTest.m_keyFrames[ g_iCurObj][ animTest.m_iCurFrame ]->m_vecTrans.x );
		SetWindowText( hEDITBOX_X, szBuffer );
		sprintf( szBuffer, "%.2f", animTest.m_keyFrames[ g_iCurObj][ animTest.m_iCurFrame ]->m_vecTrans.y );
		SetWindowText( hEDITBOX_Y, szBuffer );
		sprintf( szBuffer, "%.2f", animTest.m_keyFrames[ g_iCurObj][ animTest.m_iCurFrame ]->m_vecTrans.z );
		SetWindowText( hEDITBOX_Z, szBuffer );
		// Rotation
		sprintf( szBuffer, "%.2f", animTest.m_keyFrames[ g_iCurObj][ animTest.m_iCurFrame ]->m_vecRot.x );
		SetWindowText( hEDITBOX_XROT, szBuffer );
		sprintf( szBuffer, "%.2f", animTest.m_keyFrames[ g_iCurObj][ animTest.m_iCurFrame ]->m_vecRot.y );
		SetWindowText( hEDITBOX_YROT, szBuffer );
		sprintf( szBuffer, "%.2f", animTest.m_keyFrames[ g_iCurObj][ animTest.m_iCurFrame ]->m_vecRot.z );
		SetWindowText( hEDITBOX_ZROT, szBuffer );
	}
}

//--------------------------------------------------------------------------------------
//
// Sets up the animation to be edited.  For now, all that
// you must do is set the internal Direct3D pointer
//
//--------------------------------------------------------------------------------------
void vInitAnimation( void )
{
	// Set the 3d device
	animTest.vSet3DDevice( g_pd3dDevice );
}

//--------------------------------------------------------------------------------------
//
// Loads objects into the animation scene.  For now this
// is hardcoded.  You should modify it to load objects
// dynamically.
//
//--------------------------------------------------------------------------------------
void vLoadObject( void )
{
	//
	// LOAD THE ROBOT OBJECTS
	// *NOTE* This is hardcoded only for
	// demonstration purposes.
	//
	// *FOR YOU TO DO* Add the ability 
	// to load objects from a file 
	// requestor.
	//

	// Reset the current animation
	animTest.vReset();

	// Load the hard-coded objects
	animTest.iNewObj( "droid_still" );
	animTest.iNewObj( "radar_dish" );

	// Render the scene
	vRender();

	// Update the toolbar
	vUpdateToolbarStats();
}
