//-----------------------------------------------------------------------------
// Name: D3D_Pathfinding
//
// Description: Example code showing how to perform path finding.
//
// File Function: Main body of the program (main.cpp)
//
// Code: 
//		Copyright (c) 2003 LostLogic Corporation. All rights reserved.
//
// Libraries Required:
//		d3d9.lib dxguid.lib d3dx9dt.lib d3dxof.lib comctl32.lib winmm.lib
//
// Local Files Required:
//		main.cpp
//		main.h
//		CPathFinder.h
//		CPathFinder.cpp
//
// DX Files: 
//		Copyright (c) 1997-2001 Microsoft Corporation. All rights reserved.
//-----------------------------------------------------------------------------
#include "main.h"

//--------------------------------------------------------------------------------------
//
// Main windows function, ie main()
//
//--------------------------------------------------------------------------------------
int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance, PSTR szCmdLine, int iCmdShow)
{
	HWND		hWnd;
	MSG			msg;
	WNDCLASSEX	wndclass;
	RECT		rcWindowClient;
	DWORD		dwInputTimer = 0;
	int			iResult = 0;
		
    // Set up window class
	wndclass.cbSize			= sizeof(wndclass);
	wndclass.style			= CS_HREDRAW | CS_VREDRAW;
	wndclass.lpfnWndProc	= fnMessageProcessor;
	wndclass.cbClsExtra		= 0;
	wndclass.cbWndExtra		= 0;
	wndclass.hInstance		= hInstance;
	wndclass.hIcon			= LoadIcon( NULL, IDI_APPLICATION );
	wndclass.hCursor		= LoadCursor( NULL, IDC_ARROW );
	wndclass.hbrBackground	= (HBRUSH) GetStockObject (WHITE_BRUSH);
	wndclass.lpszMenuName	= NULL;
	wndclass.lpszClassName	= "Path Finding";	// Registered Class Name
	wndclass.hIconSm		= LoadIcon( NULL, IDI_APPLICATION );

	// Register the window class
	if( RegisterClassEx( &wndclass ) == NULL ) 
	{
		// The program failed, exit
		exit(1);
	}

	// Create the window
	hWnd = CreateWindowEx(	WS_EX_OVERLAPPEDWINDOW,
							"Path Finding",		// Class Name
							"D3D_PathFinding",		// Name Displayed on Title Bar
							WS_OVERLAPPEDWINDOW,
							0,
							0,
							g_iWindowWidth,
							g_iWindowHeight,
							NULL,
							NULL,
							hInstance,
							NULL );

	// Store window handle globally for access later
	g_hWnd		= hWnd;
	g_hInstance = hInstance;
	
	// Display the window
	ShowWindow( hWnd, iCmdShow );

	// Figure out the client work area
	GetClientRect( hWnd, &rcWindowClient );

	// Calculate the rendering offsets based on the client size
	g_iXOffset = (g_iWindowWidth-(rcWindowClient.right-rcWindowClient.left));
	g_iYOffset = (g_iWindowHeight-(rcWindowClient.bottom-rcWindowClient.top));

	// Resize the window to be truely the resolution desired
	SetWindowPos( 
				hWnd, 
				NULL, 
				0,								// X-Pos
				0,								// Y-Pos
				g_iWindowWidth + g_iXOffset,	// Width
				g_iWindowHeight + g_iYOffset,	// Height
				NULL);

	// Clear out message structure
	ZeroMemory( &msg, sizeof(msg) );

	// Initialize Direct3D
    if( SUCCEEDED( InitD3D( hWnd ) ) ) {
			
		// Initialize the virtual buffer for the display quad
		vInitInterfaceObjects();

		// Load the "default" map
		vLoadMap( "PathMap1.dat" );

		// Initialize the toolbar
		vCreateToolbar( hWnd, hInstance );

        // Enter the message loop
        while( msg.message!=WM_QUIT ) {
            if( PeekMessage( &msg, NULL, 0U, 0U, PM_REMOVE ) ) {
                TranslateMessage( &msg );
                DispatchMessage( &msg );
            }
            else {
				// Render the scene
				vRender();
			}
        }
    }
	// Clean up everything and exit the app
    vCleanup();
    UnregisterClass( "Path Finding", wndclass.hInstance );
    return 0;
}

//--------------------------------------------------------------------------------------
//
// Windows message processor
//
//--------------------------------------------------------------------------------------
LRESULT WINAPI fnMessageProcessor( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
    switch( msg )
    {
		case WM_DESTROY:
            PostQuitMessage( 0 );
            return 0;

		case WM_LBUTTONDOWN:
			g_iLeftButtonDown = 1;
			break;

		case WM_LBUTTONUP:
			g_iLeftButtonDown = 0;
			break;

		case WM_COMMAND:
				switch(LOWORD(wParam))
				{
					// Check toolbar controls
					case ID_BUTTON_LOAD:
						vLoadMap();
						break;
					case ID_BUTTON_GO:
						vInitPathing();
						break;
					default:
						break;
				}

		default:
			break;
    }
    return DefWindowProc( hWnd, msg, wParam, lParam );
}

//--------------------------------------------------------------------------------------
//
// Function to initialize Direct 3D
//
//--------------------------------------------------------------------------------------
HRESULT InitD3D( HWND hWnd )
{
	D3DPRESENT_PARAMETERS	d3dpp;
	D3DXMATRIX				matproj, matview;
	D3DDISPLAYMODE			d3ddm;

    // Create the D3D object
    if( NULL == ( g_pD3D = Direct3DCreate9( D3D_SDK_VERSION ) ) )
        return E_FAIL;
	
	// Get the current desktop display mode, so we can set up a back
	// buffer of the same format
    if( FAILED( g_pD3D->GetAdapterDisplayMode( D3DADAPTER_DEFAULT, &d3ddm ) ) )
        return E_FAIL;
	
    // Setup the back buffer and swap format
	ZeroMemory( &d3dpp, sizeof(d3dpp) );
	d3dpp.Windowed					= TRUE;
	d3dpp.SwapEffect				= D3DSWAPEFFECT_DISCARD;
	d3dpp.BackBufferFormat			= d3ddm.Format;
	d3dpp.EnableAutoDepthStencil	= FALSE;

    // Create the D3DDevice
    if( FAILED( g_pD3D->CreateDevice( D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hWnd,
                                      D3DCREATE_HARDWARE_VERTEXPROCESSING,
                                      &d3dpp, &g_pd3dDevice ) ) )
    {
        return E_FAIL;
    }
	
	//---------------------------------------------
	//     Setup 2D View and Rendering States
	//---------------------------------------------
	D3DXMatrixIdentity( &matview );
	g_pd3dDevice->SetTransform( D3DTS_VIEW, &matview );
	
	// Setup orthographic rendering, ie 2D in 3D
	D3DXMatrixOrthoLH( &matproj, (float)g_iWindowWidth, (float)g_iWindowHeight, 0, 1 );
	// Setup the projection matrix
	g_pd3dDevice->SetTransform( D3DTS_PROJECTION, &matproj );
	// Turn off culling
	g_pd3dDevice->SetRenderState( D3DRS_CULLMODE, D3DCULL_NONE );
	// Turn off lighting
	g_pd3dDevice->SetRenderState( D3DRS_LIGHTING, FALSE );
	// Turn off z-buffer
	g_pd3dDevice->SetRenderState( D3DRS_ZENABLE, FALSE );
	g_pd3dDevice->SetRenderState( D3DRS_ZWRITEENABLE, FALSE );

    return S_OK;
}

//--------------------------------------------------------------------------------------
//
// Function to cleanup allocated objects
//
//--------------------------------------------------------------------------------------
void vCleanup( void )
{
	// Free the vertex buffer
    if( g_pVBInterface != NULL )        
        g_pVBInterface->Release();

	// Free the 3D device
    if( g_pd3dDevice != NULL ) 
        g_pd3dDevice->Release();

	// Free Direct 3D
    if( g_pD3D != NULL )       
        g_pD3D->Release();
}

//--------------------------------------------------------------------------------------
//
// Function to draw the 3D scene
//
//--------------------------------------------------------------------------------------
void vRender( void )
{
	RECT	rectText;
	int		iX;
	int		iY;
	int		iLayer;
	int		iCurTile;
	int		iBufferPos;
	char	szOutput[ 256 ];

	// Clear the backbuffer to a blue color
	g_pd3dDevice->Clear( 0, NULL, D3DCLEAR_TARGET, D3DCOLOR_XRGB(0,0,0), 1.0f, 0 );

	// Begin the scene
	g_pd3dDevice->BeginScene();

	// Turn on alpha-blending
	g_pd3dDevice->SetRenderState( D3DRS_ALPHABLENDENABLE,   TRUE );
	g_pd3dDevice->SetRenderState( D3DRS_SRCBLEND,			D3DBLEND_SRCALPHA );
	g_pd3dDevice->SetRenderState( D3DRS_DESTBLEND,			D3DBLEND_INVSRCALPHA );

	//---------------------
	//
	// RENDER THE TILES
	//
	//---------------------

	// Top to bottom
	for( iY = 0; iY < g_iTilesHigh; iY++ ) {

		// Left to right
		for( iX = 0; iX < g_iTilesWide; iX++ ) {

			// Layers
			for( iLayer = 0; iLayer < 4; iLayer++ ) {
			
				// Calculate buffer offset
				iBufferPos = iX+g_iXPos+((iY+g_iYPos)*g_iMapWidth);

				// Get the proper tile
				iCurTile = g_iTileMap[ iBufferPos ][ iLayer ];
				
				// Display the tile
				// Only display tile 0 on layer 0; otherwise
				// it is a non-tile used for erasing
				if( iCurTile != 0 || iLayer == 0) {
					vDrawInterfaceObject(	( iX * g_iTileSize ), 
											( iY * g_iTileSize ), 
											(float)g_iTileSize, 
											(float)g_iTileSize, 
											g_pTexture[ iCurTile ] );
				}
			}
		}
	}

	//
	// Render the arrow map
	//
	for( iY = 0; iY < g_iTilesHigh; iY++ ) {

		// Left to right
		for( iX = 0; iX < g_iTilesWide; iX++ ) {

			// Calculate buffer offset
			iBufferPos = iX+g_iXPos+((iY+g_iYPos)*g_iMapWidth);

			// Get the proper tile
			iCurTile = g_iArrowMap[ iBufferPos ];
			
			// Display the tile
			// Only display tile 0 on layer 0; otherwise
			// it is a non-tile used for erasing
			if( iCurTile != -1 ) {
				vDrawInterfaceObject(	( iX * g_iTileSize ), 
										( iY * g_iTileSize ), 
										(float)g_iTileSize, 
										(float)g_iTileSize, 
										g_pArrowTextures[ iCurTile ] );
			}
		}
	}

	//
	// Render the rabbit
	//
	vDrawInterfaceObject(	( g_iRabbitXPos * g_iTileSize ), 
							( g_iRabbitYPos * g_iTileSize ), 
							(float)g_iTileSize, 
							(float)g_iTileSize, 
							g_pRabbitTextures[ g_iRabbitDir] );
	

	// Draw the text
	rectText.bottom		= 50;
	rectText.left		= 0;
	rectText.right		= 640;
	rectText.top		= 0;
	sprintf( 	szOutput, "%s", 
				g_szPathStatus );

	pD3DXFont->DrawText(	szOutput, -1, &rectText, 
							DT_LEFT, 
							D3DCOLOR_RGBA(255, 255, 255, 255));

	// End the scene
	g_pd3dDevice->EndScene();

	// Present the backbuffer contents to the display
	g_pd3dDevice->Present( NULL, NULL, NULL, NULL );
}

//--------------------------------------------------------------------------------------
//
// Function to initialize the global interface object vertex buffer
//
// Uses the following global variables:
//
//		g_pd3dDevice			The Direct3D Device
//		g_pVBInterface			The Vertex buffer for the interface objects
//		CUSTOMVERTEX			The Vertex structure
//		D3DFVF_CUSTOMVERTEX		The Vertex format
//		g_pTexture				The Texture array to hold textures
//
//--------------------------------------------------------------------------------------
void vInitInterfaceObjects( void )
{
	CUSTOMVERTEX	*pVertices;
	char			szTileName[ 32 ];

	// Create the interface object buffer
	if( FAILED( g_pd3dDevice->CreateVertexBuffer( 4*sizeof(CUSTOMVERTEX),
		0, D3DFVF_CUSTOMVERTEX,
		D3DPOOL_DEFAULT, &g_pVBInterface, NULL ) ) )
	{
		return;
	}

	// Lock the buffer for editing
	if( FAILED( g_pVBInterface->Lock( 0, 0, (void**)&pVertices, 0 ) ) )
		return;

	//
	// Create a quad made up of 3 vertices (triangle strips)
	//
	// 1          3
	//  X--------X
	//  |x       |
	//  | x      |
	//  |  x     |
	//  |   x    |
	//  |    x   |
	//  |     x  |
	//  |      x |
	//  |       x|
	//  X--------X
	// 0          2
	//
	pVertices[0].position = D3DXVECTOR3( 0.0f, 0.0f, 0.0f );
	pVertices[0].tu       = 0.0f;
	pVertices[0].tv       = 1.0f;
	pVertices[0].tu2       = 0.0f;
	pVertices[0].tv2       = 1.0f;
	pVertices[0].vecNorm = D3DXVECTOR3(0.0f,0.0f,1.0f);

	pVertices[1].position = D3DXVECTOR3( 0.0f, 1.0f, 0.0f );
	pVertices[1].tu       = 0.0f;
	pVertices[1].tv       = 0.0f;
	pVertices[1].tu2       = 0.0f;
	pVertices[1].tv2       = 0.0f;
	pVertices[1].vecNorm = D3DXVECTOR3(0.0f,0.0f,1.0f);

	pVertices[2].position = D3DXVECTOR3( 1.0f, 0.0f, 0.0f );
	pVertices[2].tu       = 1.0f;
	pVertices[2].tv       = 1.0f;
	pVertices[2].tu2       = 1.0f;
	pVertices[2].tv2       = 1.0f;
	pVertices[2].vecNorm = D3DXVECTOR3(0.0f,0.0f,1.0f);

	pVertices[3].position = D3DXVECTOR3( 1.0f, 1.0f, 0.0f );
	pVertices[3].tu       = 1.0f;
	pVertices[3].tv       = 0.0f;
	pVertices[3].tu2       = 1.0f;
	pVertices[3].tv2       = 0.0f;
	pVertices[3].vecNorm = D3DXVECTOR3(0.0f,0.0f,1.0f);

	// Done editing, unlock the buffer
	g_pVBInterface->Unlock();

	//---------------------------------------------
	//      LOAD TEXTURES
	//---------------------------------------------

	// Load the tiles
	for( int i = 0; i < g_iTotalTiles; i++ ) {
		// Set the name
		sprintf( szTileName, "tile%d.tga", i );
		// Load it
		if( FAILED( D3DXCreateTextureFromFile( 
			g_pd3dDevice, 
			szTileName, 
			&g_pTexture[ i ] ) ) ) {

			return;
		}
	}

	// Load the rabbit textures
	for( i = 0; i < 8; i++ ) {
		// Set the name
		sprintf( szTileName, "rabbit%d.tga", i );
		// Load it
		if( FAILED( D3DXCreateTextureFromFile( 
			g_pd3dDevice, 
			szTileName, 
			&g_pRabbitTextures[ i ] ) ) ) {

			return;
		}
	}

	// Load the arrow textures
	for( i = 0; i < 8; i++ ) {
		// Set the name
		sprintf( szTileName, "arrow%d.tga", i );
		// Load it
		if( FAILED( D3DXCreateTextureFromFile( 
			g_pd3dDevice, 
			szTileName, 
			&g_pArrowTextures[ i ] ) ) ) {

			return;
		}
	}
	

	// Text Font
	hFont = CreateFont( 
		16, 0, 0, 0, 
		0, 0, 0, 0, 0, 
		0, 0, PROOF_QUALITY, 
		0, "fixedsys" ); 

	// Create the Direct3D Font
	D3DXCreateFont( g_pd3dDevice, hFont, &pD3DXFont ); 
};
//--------------------------------------------------------------------------------------
//
// Function to draw a 3D object in 2D space
//
// iXPos    = x-position in screen-space of the top-left corner of the object
// iYPos    = y-position in screen-space of the top-left corner of the object
// fXSize   = Width of the object in screen-space (pixels)
// fYSize   = Height of the object in screen-space (pixels)
// iTexture = Index into the texture array of object to draw
//
//--------------------------------------------------------------------------------------
void vDrawInterfaceObject( int iXPos, int iYPos, float fXSize, float fYSize, LPDIRECT3DTEXTURE9 tex )
{
	D3DXMATRIX	matWorld, matRotation, matTranslation, matScale;
	float		fXPos, fYPos;

	// Set default position,scale,rotation
	D3DXMatrixIdentity( &matTranslation );
	// Scale the sprite
	D3DXMatrixScaling( &matScale, fXSize, fYSize, 1.0f );
	D3DXMatrixMultiply( &matTranslation, &matTranslation, &matScale );
	// Rotate the sprite
	D3DXMatrixRotationZ( &matRotation, 0.0f );
	D3DXMatrixMultiply( &matWorld, &matTranslation, &matRotation );

	// Calculate the position in screen-space
	fXPos = (float)(-(g_iWindowWidth/2)+iXPos);
	fYPos = (g_iWindowHeight/2)-fYSize-iYPos;

	// Move the sprite
	matWorld._41 = fXPos;	// X
	matWorld._42 = fYPos;	// Y

	// Set matrix
	g_pd3dDevice->SetTransform( D3DTS_WORLD, &matWorld );
	g_pd3dDevice->SetTexture( 0, tex );
	g_pd3dDevice->SetStreamSource( 0, g_pVBInterface, 0, sizeof( CUSTOMVERTEX ) );
	g_pd3dDevice->SetFVF( D3DFVF_CUSTOMVERTEX );
	g_pd3dDevice->DrawPrimitive( D3DPT_TRIANGLESTRIP, 0, 2 );

	// Dereference texture
	g_pd3dDevice->SetTexture(0, NULL);
}

//--------------------------------------------------------------------------------------
//
// Function to initialize the map
//
//--------------------------------------------------------------------------------------
void vInitMap( void )
{
	int i, j;
	
	// Clear the map
	for( i = 0; i < g_iMapWidth * g_iMapHeight; i++ ) {
		// Base layer gets water
		g_iTileMap[ i ][ 0 ] = 1;
		// Other layers get 0
		for( j = 1; j < 4; j++ ) {
			g_iTileMap[ i ][ j ] = 0;
		}
	}

	// Clear the arrow map, it uses -1 for the 
	// "blank" tile
	for( i = 0; i < g_iMapWidth * g_iMapHeight; i++ ) {
		g_iArrowMap[ i ] = -1;
	}
}

//-----------------------------------------------------------------------------
//
// Creates the command toolbar
//
// Commands
// --------
// Load - Loads a map created with the D3D_MapEditorLayers program
// Go   - Calculates and executes the path to solve the map loaded
//
//-----------------------------------------------------------------------------
void vCreateToolbar(HWND hwnd, HINSTANCE hinst)
{
	WNDCLASSEX	wcToolBar;

	// Set up and register toolbar window class
	wcToolBar.cbSize		= sizeof(wcToolBar);
	wcToolBar.style			= CS_HREDRAW | CS_VREDRAW;
	wcToolBar.lpfnWndProc	= fnMessageProcessor;
	wcToolBar.cbClsExtra	= 0;
	wcToolBar.cbWndExtra	= 0;
	wcToolBar.hInstance		= hinst;
	wcToolBar.hIcon			= LoadIcon( NULL, IDI_APPLICATION );
	wcToolBar.hCursor		= LoadCursor( NULL, IDC_ARROW );
	wcToolBar.hbrBackground	= (HBRUSH) GetStockObject (COLOR_BACKGROUND);
	wcToolBar.lpszMenuName	= NULL;
	wcToolBar.lpszClassName	= "ToolBar";	// Registered Class Name
	wcToolBar.hIconSm		= LoadIcon( NULL, IDI_APPLICATION );
    RegisterClassEx(&wcToolBar);
    
	// Create toolbar window
	hWndToolBar = CreateWindowEx ( 
		WS_EX_LEFT|WS_EX_TOPMOST|WS_EX_TOOLWINDOW,
		"ToolBar", "Command Bar",
		WS_BORDER | WS_VISIBLE | WS_CAPTION | WS_MINIMIZEBOX,	
		g_iWindowWidth-100, g_iYOffset, 100, 80, hwnd, NULL, hinst, NULL);

	// Load Button
	hBUTTON_LOAD = CreateWindow(
		"BUTTON", "Load",
		WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON,
		5, 5, 85, 20, hWndToolBar, (HMENU)ID_BUTTON_LOAD, hinst, NULL);

	// Go Button
	hBUTTON_GO = CreateWindow(
		"BUTTON", "Go",
		WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON,
		5, 35, 85, 20, hWndToolBar, (HMENU)ID_BUTTON_GO, hinst, NULL);

	// Activate edit area
	SetActiveWindow( g_hWnd );
}

//-----------------------------------------------------------------------------
//
// Loads the tile map from a binary file.
//
//-----------------------------------------------------------------------------
void vLoadMap( char *szMapName )
{
	FILE			*fp;
	OPENFILENAME	fileStruct;
	char			szFileName[ 512 ];
	char			szFilter[ 32 ];
	char			szExtension[ 32 ];
	int				iRet;

	// Clear out maps prior to loading
	vInitMap();

	// Clear buffer to receive file name
	memset( szFileName, 0x00, 512 );

	// Create file filter
	memset( szFilter, 0x00, 32 );
	strcpy( szFilter, "*.dat" );

	// Create file extension
	memset( szExtension, 0x00, 32 );
	strcpy( szExtension, "dat" );

	//
	// Setup the file dialog box
	//

	if( szMapName == NULL ) {

		// Clear file dialog structure
		memset( &fileStruct, 0x00, sizeof( OPENFILENAME ) );

		// Initialize structure
		fileStruct.hInstance = g_hInstance;
		fileStruct.hwndOwner = g_hWnd;
		fileStruct.lpstrDefExt = szExtension;
		fileStruct.lpstrFileTitle = szFileName;
		fileStruct.lpstrFilter = szFilter;
		fileStruct.nMaxFileTitle = 512;
		fileStruct.lStructSize = sizeof( OPENFILENAME );

		// Retrieve the filename
		iRet = GetOpenFileName( &fileStruct );

		// Exit out on failure
		if( !iRet ) {
			return;
		}
	}
	else {
		strcpy( szFileName, szMapName );
	}

	// Open the tile file
	fp = fopen( szFileName, "rb" );

	// Return if no file to load
	if( fp == NULL ) {
		return;
	}
	
	// Read the tile buffer
	fread( g_iTileMap, 40000, sizeof(int), fp );

	// Close the tile file
	fclose( fp );
	
	// Play sound to indicate action
	PlaySound("button.wav",NULL,SND_FILENAME|SND_ASYNC);
}

//------------------------------------------------------------------------
//
//	PATH FINDING FUNCTIONS
//
//------------------------------------------------------------------------

//
// Function to get the map cost
//
// >0 = passable
// -1 = blocked
//
int iGetMapCost( int iX, int iY )
{
	// If out of horizontal bounds return impassable
	if( iX < 0 || iX >= g_iTilesWide ) 
		return( -1 );

	// If out of vertical bounds return impassable
	if( iY < 0 || iY >= g_iTilesHigh ) 
		return( -1 );

	// Return impassable if not the 0 tile
	if( g_iTileMap[ iX+(iY*g_iMapWidth) ][ 1 ] != 0 ) {
		return( -1 );
	}
	// Return tile value for anything else
	else {
		return( g_iTileMap[ iX+(iY*g_iMapWidth) ][ 1 ] );
	}
}

//
// Initialize the path and follow it
//
void vInitPathing( void )
{
	bool			bRet;
	int				iTempX;
	int				iTempY;
	int				iDir;
	// Start & End map positions
	int				iNodeStartX;
	int				iNodeStartY;
	int				iNodeEndX;
	int				iNodeEndY;
	// Timers
	DWORD			dwStartTime;
	DWORD			dwTotalTime;
	// Path class object
	CPathFinder		pathMyPath;

	// Clear the arrow map
	// Use it later to show the path
	for( int i = 0; i < g_iMapWidth * g_iMapHeight; i++ ) {
		g_iArrowMap[ i ] = -1;
	}
	
	//
	// Set rabbit start pos
	//

	// Search the map for a "start" tile
	for( int y = 0; y < g_iMapHeight; y++ ) {
		for( int x = 0; x < g_iMapWidth; x++ ) {
			if( g_iTileMap[ x+(y*g_iMapWidth) ][ 0 ] == 19 ) {
				g_iRabbitXPos = x;
				g_iRabbitYPos = y;
				// Create a start state
				iNodeStartX = g_iRabbitXPos;
				iNodeStartY = g_iRabbitYPos; 
				break;
			}
		}
	}

	//
	// Set rabbit end pos
	//

	// Search the map for a "end" tile
	for( y = 0; y < g_iMapHeight; y++ ) {
		for( int x = 0; x < g_iMapWidth; x++ ) {
			if( g_iTileMap[ x+(y*g_iMapWidth) ][ 0 ] == 20 ) {
				iNodeEndX = x;							
				iNodeEndY = y; 
				break;
			}
		}
	}

	// Update rendered message
	sprintf( g_szPathStatus, "CALCULATING PATH" );
	vRender();

	// Setup the cost function
	pathMyPath.vSetCostFunction( iGetMapCost );
	// Start the timer
	dwStartTime = timeGetTime();
	// Setup the start and end
	pathMyPath.vSetStartState( iNodeStartX, iNodeStartY, iNodeEndX, iNodeEndY );
	// Find the path - 300 maximum nodes
	bRet = pathMyPath.bFindPath( 300 );
	// Stop the timer
	dwTotalTime = timeGetTime()-dwStartTime;

	// Exit on failure
	if( !bRet ) {
		// Update rendered message
		sprintf(	g_szPathStatus, 
					"FAILED, OPEN = %d, CLOSED = %d, TIME = %ld",
					pathMyPath.m_iActiveOpenNodes, 
					pathMyPath.m_iActiveClosedNodes,
					dwTotalTime );
		return;
	}
	else {
		// Update rendered message
		sprintf(	g_szPathStatus, 
					"COMPLETE, OPEN = %d, CLOSED = %d, TIME = %ld",
					pathMyPath.m_iActiveOpenNodes, 
					pathMyPath.m_iActiveClosedNodes,
					dwTotalTime );
	}

	// Follow the path now
	CPathNode	*GoalNode = pathMyPath.m_CompletePath->m_Path[ 0 ];
	int			iTotalNodes = 0;

	// Set temp position to figure out direction arrow
	iTempX = GoalNode->m_iX;
	iTempY = GoalNode->m_iY;

	// Start at position 1 not 0
	iTotalNodes++;
	GoalNode = pathMyPath.m_CompletePath->m_Path[ iTotalNodes ];

	// Loop through the path and follow it
	// Draw an arrow for each step
	while( iTotalNodes < pathMyPath.m_CompletePath->m_iNumNodes ) {
		// Figure out direction
		iDir = vFindDirection( iTempX, iTempY, GoalNode->m_iX, GoalNode->m_iY );

		// Set arrow on arrow map
		g_iArrowMap[ GoalNode->m_iX+(GoalNode->m_iY*g_iMapWidth) ] = iDir;

		// Render the scene
		vRender();
		
		// Set temp position to figure out direction arrow
		iTempX = GoalNode->m_iX;
		iTempY = GoalNode->m_iY;

		// Increment node count
		iTotalNodes++;

		// Get next node
		GoalNode = pathMyPath.m_CompletePath->m_Path[ iTotalNodes ];
	};
}

//
// Figure out which direction to face when moving
// from a start position to an end position
//
int vFindDirection( int iStartX, int iStartY, int iEndX, int iEndY )
{
	int iDir;

	if( iEndX > iStartX ) {
		if( iEndY > iStartY ) {
			iDir = 3;
		}
		else if ( iEndY == iStartY ) {
			iDir = 2;
		}
		else if ( iEndY < iStartY ) {
			iDir = 1;
		}
	}
	else if( iEndX < iStartX ) {
		if( iEndY > iStartY ) {
			iDir = 5;
		}
		else if ( iEndY == iStartY ) {
			iDir = 6;
		}
		else if ( iEndY < iStartY ) {
			iDir = 7;
		}
	}
	if( iEndX == iStartX ) {
		if( iEndY > iStartY ) {
			iDir = 4;
		}
		else if ( iEndY == iStartY ) {
			iDir = 2;
		}
		else if ( iEndY < iStartY ) {
			iDir = 0;
		}
	}

	return( iDir );
}
